{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYDelphi5Lexer;

interface

uses { vcl } SysUtils,                                           
     { ly  } LexLib,
     {  oy } OYLexLib, OYDelphi5Parser;

  function TokenKind(TokenText : String) : Integer;

{$I OYDelphi5Lex.inc}

{ helper function to convert token strings into token
  numeric ids }
function TokenKind(TokenText : String) : Integer;
var
  TokenHash : Integer;
  aToken : String;
begin
  Result := tkIdentifier;
  aToken := UpperCase(TokenText);
  TokenHash := OYGetTokenHash(aToken);
  Case TokenHash of
    143: if aToken = 'IF' then Result := tkIf;
    147: if aToken = 'DO' then Result := tkDo;
    148: if aToken = 'AS' then Result := tkAs;
    149:
      begin
        if aToken = 'OF' then Result := tkOf;
        if aToken = 'AT' then Result := tkAt;
      end;
    151: if aToken = 'IN' then Result := tkIn;
    156: if aToken = 'IS' then Result := tkIs;
    157: if aToken = 'ON' then Result := tkOn;
    161: if aToken = 'OR' then Result := tkOr;
    163: if aToken = 'TO' then Result := tkTo;
    211: if aToken = 'AND' then Result := tkAnd;
    215: if aToken = 'END' then Result := tkEnd;
    217: if aToken = 'FAR' then Result := tkFar;
    224: if aToken = 'MOD' then Result := tkMod;
    227:
      begin
        if aToken = 'DIV' then Result := tkDiv;
        if aToken = 'NIL' then Result := tkNil;
      end;
    231:
      begin
        if aToken = 'FOR' then Result := tkFor;
        if aToken = 'SHL' then Result := tkShl;
      end;
    233: if aToken = 'VAR' then Result := tkVar;
    236: if aToken = 'SET' then Result := tkSet;
    237: if aToken = 'SHR' then Result := tkShr;
    241: if aToken = 'NOT' then Result := tkNot;
    248: if aToken = 'OUT' then Result := tkOut;
    249: if aToken = 'XOR' then Result := tkXor;
    255: if aToken = 'TRY' then Result := tkTry;
    284:
      begin
        if aToken = 'CASE' then Result := tkCase;
        if aToken = 'READ' then Result := tkRead;
      end;
    286: if aToken = 'CHAR' then Result := tkChar;
    288: if aToken = 'FILE' then Result := tkFile;
    289: if aToken = 'NAME' then Result := tkName;
    292: if aToken = 'REAL' then Result := tkReal;
    294: if aToken = 'NEAR' then Result := tkNear;
    297: if aToken = 'ELSE' then Result := tkElse;
    303:
      begin
        if aToken = 'THEN' then Result := tkThen;
        if aToken = 'COMP' then Result := tkComp;
      end;
    308: if aToken = 'BYTE' then Result := tkByte;
    313: if aToken = 'GOTO' then Result := tkGoto;
    316:
      begin
        if aToken = 'WITH' then Result := tkWith;
        if aToken = 'WORD' then Result := tkWord;
      end;
    320:
      begin
        if aToken = 'UNIT' then Result := tkUnit;
        if aToken = 'USES' then Result := tkUses;
      end;
    322: if aToken = 'TYPE' then Result := tkType;
    341: if aToken = 'INT64' then Result := tkInt64;
    347: if aToken = 'CDECL' then Result := tkCdecl;
    352: if aToken = 'LABEL' then Result := tkLabel;
    357: if aToken = 'BEGIN' then Result := tkBegin;
    366: if aToken = 'PCHAR' then Result := tkPchar;
    372: if aToken = 'RAISE' then Result := tkRaise;
    374: if aToken = 'CLASS' then Result := tkClass;
    376: if aToken = 'INDEX' then Result := tkIndex;
    377: if aToken = 'WHILE' then Result := tkWhile;
    383: if aToken = 'ARRAY' then Result := tkArray;
    391: if aToken = 'CONST' then Result := tkConst;
    395: if aToken = 'WRITE' then Result := tkWrite;
    396: if aToken = 'UNTIL' then Result := tkUntil;
    400: if aToken = 'REAL48' then Result := tkReal48;
    424: if aToken = 'PACKED' then Result := tkPacked;
    436: if aToken = 'PASCAL' then Result := tkPascal;
    439: if aToken = 'OBJECT' then Result := tkObject;
    443: if aToken = 'DOUBLE' then Result := tkDouble;
    445: if aToken = 'DISPID' then Result := tkDispid;
    447:
      begin
        if aToken = 'INLINE' then Result := tkInline;
        if aToken = 'PUBLIC' then Result := tkPublic;
        if aToken = 'RECORD' then Result := tkRecord;
      end;
    449: if aToken = 'REPEAT' then Result := tkRepeat;
    450: if aToken = 'SINGLE' then Result := tkSingle;
    457: if aToken = 'EXCEPT' then Result := tkExcept;
    465: if aToken = 'STORED' then Result := tkStored;
    471: if aToken = 'STRING' then Result := tkString;
    475: if aToken = 'DOWNTO' then Result := tkDownto;
    482: if aToken = 'EXPORT' then Result := tkExport;
    492: if aToken = 'PACKAGE' then Result := tkPackage;
    512: if aToken = 'BOOLEAN' then Result := tkBoolean;
    517:
      begin
        if aToken = 'DEFAULT' then Result := tkDefault;
        if aToken = 'DYNAMIC' then Result := tkDynamic;
        if aToken = 'MESSAGE' then Result := tkMessage;
      end;
    519: if aToken = 'STDCALL' then Result := tkStdcall;
    527: if aToken = 'FINALLY' then Result := tkFinally;
    526: if aToken = 'INTEGER' then Result := tkOrdInteger;
    533:
      begin
        if aToken = 'FORWARD' then Result := tkForward;
        if aToken = 'LIBRARY' then Result := tkLibrary;
        if aToken = 'VARIANT' then Result := tkVariant;
      end;
    536: if aToken = 'PROGRAM' then Result := tkProgram;
    539:
      begin
        if aToken = 'PRIVATE' then Result := tkPrivate;
        if aToken = 'LONGINT' then Result := tkLongint;
      end;
    551: if aToken = 'VIRTUAL' then Result := tkVirtual;
    565: if aToken = 'EXPORTS' then Result := tkExports;
    571: if aToken = 'SAFECALL' then Result := tkSafecall;
    574: if aToken = 'CARDINAL' then Result := tkCardinal;
    583: if aToken = 'WIDECHAR' then Result := tkWideChar;
    593: if aToken = 'EXTENDED' then Result := tkExtended;
    596: if aToken = 'ABSTRACT' then Result := tkAbstract;
    604: if aToken = 'OVERLOAD' then Result := tkOverload;
    606:
      begin
        if aToken = 'READONLY' then Result := tkReadonly;
        if aToken = 'RESIDENT' then Result := tkResident;
      end;
    607:
      begin
        if aToken = 'ABSOLUTE' then Result := tkAbsolute;
        if aToken = 'CONTAINS' then Result := tkContains;
      end;
    608: if aToken = 'OVERRIDE' then Result := tkOverride;
    611: if aToken = 'EXTERNAL' then Result := tkExternal;
    612: if aToken = 'SMALLINT' then Result := tkSmallInt;
    613: if aToken = 'REGISTER' then Result := tkRegister;
    614: if aToken = 'FUNCTION' then Result := tkFunction;
    619: if aToken = 'CURRENCY' then Result := tkCurrency;
    620: if aToken = 'LONGWORD' then Result := tkLongword;
    624: if aToken = 'REQUIRES' then Result := tkRequires;
    635: if aToken = 'SHORTINT' then Result := tkShortInt;
    645: if aToken = 'PROPERTY' then Result := tkProperty;
    657: if aToken = 'INTERFACE' then Result := tkInterface;
    668: if aToken = 'INHERITED' then Result := tkInherited;
    670: if aToken = 'ASSEMBLER' then Result := tkAssembler;
    672: if aToken = 'PUBLISHED' then Result := tkPublished;
    673: if aToken = 'THREADVAR' then Result := tkThreadvar;
    674: if aToken = 'NODEFAULT' then Result := tkNodefault;
    676: if aToken = 'AUTOMATED' then Result := tkAutomated;
    681: if aToken = 'PROCEDURE' then Result := tkProcedure;
    682: if aToken = 'PROTECTED' then Result := tkProtected;
    717: if aToken = 'WRITEONLY' then Result := tkWriteonly;
    757: if aToken = 'OLEVARIANT' then Result := tkOleVariant;
    766: if aToken = 'IMPLEMENTS' then Result := tkImplements;
    768: if aToken = 'WIDESTRING' then Result := tkWideString;
    770: if aToken = 'ANSISTRING' then Result := tkAnsiString;
    783: if aToken = 'DESTRUCTOR' then Result := tkDestructor;
    836: if aToken = 'REINTRODUCE' then Result := tkReintroduce;
    870: if aToken = 'CONSTRUCTOR' then Result := tkConstructor;
    904: if aToken = 'FINALIZATION' then Result := tkFinalization;
    961: if aToken = 'DISPINTERFACE' then Result := tkDispinterface;
    1062: if aToken = 'IMPLEMENTATION' then Result := tkImplementation;
    1064: if aToken = 'INITIALIZATION' then Result := tkInitialization;
    1087:
      begin
        if aToken = 'RESOURCESTRING' then Result := tkResourceString;
        if aToken = 'STRINGRESOURCE' then Result := tkStringResource;
      end;
  end;
end;

end.
 