{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}

library bop130;

{$E llp}
{$R *.RES}

uses
  Windows,
  SysUtils,
  Classes,
  OYApplication,
  OYPackage,
  OYProcessingInfoF,
  OYYaccLib,
  OYDelphi5Producer,
  OYDelphi5Lexer;

const
  cPackageGuid = '{BC62B507-6CB1-4E96-90D3-2FEAE2EAA253}';
  cCopyRight = 'Copyright (C) 2000-2001 Outplay Inc.';
  cBaseLanguageVersion = '13.0';
  cBaseLanguage = 'Borland Delphi 5 Object Pascal';
  cBaseLanguageDescription = '"' + cBaseLanguage + '" is a general purpose programming language implemented in Borland Delphi and Kylix.';
  cFileNameFilter = cBaseLanguage + ' Files (*.PAS; *.DPK; *.DPR)|*.PAS;*.DPK;*.DPR';

type
  { package for Object Pascal 13.0 language processing }
  TOYBorlandObjectPascal130Package = class(TInterfacedObject, IOYLinguineLanguagePackage)
  private
    fIYaccSchema : IOYLinguineSchema;
    fILexSchema : IOYLinguineSchema;
  protected
    function PackageGuid : WideString;
    function PackageCopyright : WideString;
    function BaseLanguageVersion : WideString;
    function BaseLanguage : WideString;
    function BaseLanguageDescription : WideString;
    function FileNameFilter : WideString;
    function ILexSchema : IOYLinguineSchema;
    function IYaccSchema : IOYLinguineSchema;
    function CreateSyntaxHighlighterForUrl(
      const Url : WideString;
      const IContext : IOYLinguineExecutionContext;
      const IOptions : IOYLinguineProcessingOptions;
      out IHighlighter : IOYSyntaxHighlight;
      out   IInfo : IOYLinguineProcessingInfo
    ) : Boolean;
    function ProduceFullXmlTreeForUrl(
      const Url : WideString;
      const IContext : IOYLinguineExecutionContext;
      const IOptions : IOYLinguineProcessingOptions;
      const IConsumer : IOYLinguineConsumer;
      out   IInfo : IOYLinguineProcessingInfo
    ) : Boolean;
  public
    constructor Create;
    destructor Destroy; override;

  end;

{ globals }
function OYCreatePackageInstance(const IContext : IOYLinguineExecutionContext; out IPackage : IOYLinguineLanguagePackage) : HResult; stdcall;
begin
  IPackage := TOYBorlandObjectPascal130Package.Create;
  Result := S_OK;
end;

{ factory for linguine package }
exports OYCreatePackageInstance name cOYPackageFactoryMethodName;

{ TOYLinguinePackagePascal }
constructor TOYBorlandObjectPascal130Package.Create;
begin
  inherited Create;
  fIYaccSchema := TOYYaccLinguineSchema.Create(TOYDelphi5TemplatedParser);
  fILexSchema := TOYLexLinguineSchema.Create(TOYDelphi5TemplatedParser);
end;

destructor TOYBorlandObjectPascal130Package.Destroy;
begin
  inherited Destroy;
end;

function TOYBorlandObjectPascal130Package.CreateSyntaxHighlighterForUrl(
  const Url : WideString;
  const IContext : IOYLinguineExecutionContext;
  const IOptions : IOYLinguineProcessingOptions;
  out IHighlighter : IOYSyntaxHighlight;
  out   IInfo : IOYLinguineProcessingInfo
) : Boolean;
begin
  IInfo := nil;
  try
    IHighlighter := TOYDelphi5Highlighter.Create(Self, Url, IContext, IOptions, TTemplatedLexer);
    Result := True;
  except
    on E : Exception do begin
      IInfo := TOYProcessingInfo.Create(icError, 1, Url, E.Message, '', '', -1, -1, -1, IInfo);
      Result := False;
    end;
  end;
end;

function TOYBorlandObjectPascal130Package.ProduceFullXmlTreeForUrl(
  const Url : WideString;
  const IContext : IOYLinguineExecutionContext;
  const IOptions : IOYLinguineProcessingOptions;
  const IConsumer : IOYLinguineConsumer;
  out   IInfo : IOYLinguineProcessingInfo
) : Boolean;

var
  aProducer : TOYDelphi5Producer;
begin
  IInfo := nil;
  try
    aProducer := TOYDelphi5Producer.Create(IContext, IConsumer);
    try
      Result := aProducer.Execute(Url);
      IInfo := aProducer.GetProcessingInfo;
    finally
      aProducer.Free;
    end;
  except
    on E : Exception do begin
      IInfo := TOYProcessingInfo.Create(icError, 1, Url, E.Message, '', '', -1, -1, -1, IInfo);
      Result := False;
    end;
  end;
end;

function TOYBorlandObjectPascal130Package.PackageCopyright : WideString;
begin
  Result := cCopyRight;
end;

function TOYBorlandObjectPascal130Package.PackageGuid : WideString;
begin
  Result := cPackageGuid;
end;

function TOYBorlandObjectPascal130Package.BaseLanguageVersion : WideString;
begin
  Result := cBaseLanguageVersion;
end;

function TOYBorlandObjectPascal130Package.BaseLanguage : WideString;
begin
  Result := cBaseLanguage;
end;

function TOYBorlandObjectPascal130Package.BaseLanguageDescription : WideString;
begin
  Result := cBaseLanguageDescription;
end;

function TOYBorlandObjectPascal130Package.FileNameFilter : WideString;
begin
  Result := cFileNameFilter;
end;
                                                        
function TOYBorlandObjectPascal130Package.ILexSchema : IOYLinguineSchema;
begin
  Result := fILexSchema;
end;

function TOYBorlandObjectPascal130Package.IYaccSchema : IOYLinguineSchema;
begin
  Result := fIYaccSchema;
end;

begin

end.

