{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}    

unit OYPackageF;

interface

uses { oy } OYApplication, OYPackage;

  { factory method to create package from the dynamic link library }
  function OYCreateInstance(
    const LibraryName : WideString; const IContext : IOYLinguineExecutionContext
  ) :  IOYLinguineLanguagePackage;

  { destructor method fro the package }
  procedure OYDestroyInstance(var IPackage : IOYLinguineLanguagePackage);
                                                                          
implementation

uses { vcl } SysUtils, Windows, Classes;

var                                               
  gCriticalSection : TRTLCriticalSection;
  gDllHandles : TStrings;

function GetLibraryFullName(BaseName : String) : String;
var
   aSz : Integer;
   aModule : string;
begin                                      
  if ExtractFilePath(BaseName) <> '' then begin
    REsult := BaseName;
  end else begin
    aSz := MAX_PATH;
    SetLength(aModule, aSz);
    aSz := GetModuleFileName(hInstance, @aModule[1], aSz);
    if aSz <> 0 then begin
      SetLength(aModule, aSz);
      Result := ExtractFilePath(aModule) + BaseName;
    end else begin
      Result := '';
    end;
  end;
end;

procedure OYDestroyInstance(var IPackage : IOYLinguineLanguagePackage);
begin
  IPackage := nil;
end;

function OYCreateInstance(
  const LibraryName : WideString;
  const IContext : IOYLinguineExecutionContext
) : IOYLinguineLanguagePackage;
var
  DllMethod : TOYPackageLoadingMethod;
  aResult : HResult;
  gDllHandle : THandle;
  aIndex : Integer;
begin
  { load DLL }                              
  aIndex := gDllHandles.IndexOf(LibraryName);
  if (aIndex <> -1) then begin
    gDllHandle := THandle(gDllHandles.Objects[aIndex]);
  end else begin
    gDllHandle := LoadLibrary(PChar(GetLibraryFullName(LibraryName)));
  end;                                                      
  if (gDllHandle <> 0) then begin
    gDllHandles.AddObject(LibraryName, Pointer(gDllHandle));
  end else begin
    raise Exception.Create('Unable to load dynamic library (' + LibraryName + ').');
  end;

  { find method address }
  @DllMethod := GetProcAddress(gDllHandle, cOYPackageFactoryMethodName);
  if @DllMethod = nil then begin
    raise Exception.Create('Dynamic library did not contain expected method (' + LibraryName + ', ' + cOYPackageFactoryMethodName + ').');
  end;
                                                                             
  { execute method }
  aResult := DllMethod(IContext, Result);
  if Failed(aResult) then begin
    raise Exception.Create('Error while accessing dynamic library content (' + LibraryName + ', ' + IntToHex(aResult, 8) + ').');
  end;
end;

initialization
  IsMultithread := True;
  InitializeCriticalSection(gCriticalSection);
  gDllHandles := TStringList.Create;

finalization                                            
  try
    while (gDllHandles.Count > 0) do begin
      FreeLibrary(THandle(gDllHandles.Objects[0]));
      gDllHandles.Delete(0);  
    end;
  finally
    gDllHandles.Free;
  end;
  DeleteCriticalSection(gCriticalSection);

end.
