{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYProcessingInfoF;

interface

uses { oy } OYApplication, OYPackage;

type
  TOYProcessingInfo = class(TInterfacedObject, IOYLinguineProcessingInfo)
  private
    fCategory : TOYLinguineProcessingInfoCategory;
    fErrorCode : Integer;
    fUrl : WideString;
    fReason : WideString;
    fSrcText : WideString;
    fPos : Integer;
    fLine : WideString;
    fLineNo : Integer;
    fLinePos : Integer;
    fIPrevInfo, fINextInfo : IOYLinguineProcessingInfo;

  protected
    function GetCategory : TOYLinguineProcessingInfoCategory;
    function GetErrorCode: Integer;
    function GetUrl: WideString;
    function GetReason: WideString;
    function GetSrcText: WideString;
    function GetPos: Integer;
    function GetLine : WideString;
    function GetLineNo: Integer;
    function GetLinePos: Integer;
    function GetFirstInfo : IOYLinguineProcessingInfo;
    function GetLastInfo : IOYLinguineProcessingInfo;
    function GetNextInfo : IOYLinguineProcessingInfo;
    function GetPrevInfo : IOYLinguineProcessingInfo;
    procedure AttachInfo(const IInfo : IOYLinguineProcessingInfo);

  public
    constructor Create(
      Category : TOYLinguineProcessingInfoCategory;
      ErrorCode : Integer;
      Url : WideString;
      Reason : WideString;
      Line : WideString;
      SrcText : WideString;
      Pos : Integer;
      LineNo : Integer;
      LinePos : Integer;
      IPreviousInfo : IOYLinguineProcessingInfo  
    );

  end;

implementation

constructor TOYProcessingInfo.Create(
  Category : TOYLinguineProcessingInfoCategory;
  ErrorCode : Integer;
  Url : WideString;
  Reason : WideString;
  Line : WideString;
  SrcText : WideString;
  Pos : Integer;
  LineNo : Integer;
  LinePos : Integer;
  IPreviousInfo : IOYLinguineProcessingInfo
);
begin
  inherited Create;

  fCategory := Category;
  fErrorCode := ErrorCode;
  fUrl := Url;
  fReason := Reason;
  fSrcText := SrcText;
  fPos := Pos;
  fLine := Line;                                                       
  fLineNo := LineNo;
  fLinePos := LinePos;

  fIPrevInfo := IPreviousInfo;                         
  fINextInfo := nil;

  if Assigned(fIPrevInfo) then begin
    fIPrevInfo.AttachInfo(Self);
  end;
end;

function TOYProcessingInfo.GetCategory : TOYLinguineProcessingInfoCategory;
begin
  Result := fCategory;
end;

function TOYProcessingInfo.GetErrorCode: Integer;
begin
  Result := fErrorCode;
end;

function TOYProcessingInfo.GetUrl: WideString;
begin
  Result := fUrl;
end;

function TOYProcessingInfo.GetReason: WideString;
begin
  Result := fReason;
end;

function TOYProcessingInfo.GetSrcText: WideString;
begin
  Result := fSrcText;
end;

function TOYProcessingInfo.GetLine: WideString;
begin
  Result := fLine;
end;

function TOYProcessingInfo.GetLineNo: Integer;
begin
  Result := fLineNo;
end;

function TOYProcessingInfo.GetLinepos: Integer;
begin
  REsult := fLinePos;
end;

function TOYProcessingInfo.Getpos: Integer;
begin
  Result := fPos;
end;

function TOYProcessingInfo.GetNextInfo : IOYLinguineProcessingInfo;
begin
  Result := fINextInfo;
end;

function TOYProcessingInfo.GetPrevInfo : IOYLinguineProcessingInfo;
begin
  Result := fIPrevInfo;
end;

function TOYProcessingInfo.GetFirstInfo : IOYLinguineProcessingInfo;
begin
  Result := Self;
  while Assigned(Result) do begin
    if not Assigned(Result.GetPrevInfo) then begin
      break;
    end;
    Result := Result.GetPrevInfo;
  end;
end;

function TOYProcessingInfo.GetLastInfo : IOYLinguineProcessingInfo;
begin
  Result := Self;
  while Assigned(Result) do begin
    if not Assigned(Result.GetNextInfo) then begin
      break;
    end;
    Result := Result.GetNextInfo;
  end;
end;

procedure TOYProcessingInfo.AttachInfo(const IInfo : IOYLinguineProcessingInfo);
var
  aO : IOYLinguineProcessingInfo;
begin
  aO := fINextInfo;
  fINextInfo := IInfo;
  if Assigned(aO) then begin
    IInfo.AttachInfo(aO)  
  end;
end;

end.
