{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}

unit OYUtility;

interface
                                                                             
uses { vcl } Windows, Classes, Graphics, Menus, SysUtils, Controls, ExtCtrls;

  {
    all functions here are to operate in stateless mode;
    no classes or interfaces are allowed in here;
    only pure API functions and procedures are allowed
  }

type
  { defined in Windows.pas }
  TRGBQuad = packed record
      r: Byte;
      g: Byte;
      b: Byte;
      rgbReserved: Byte;
  end;
  PRGBQuad = ^TRGBQuad;

  { defined in Graphics.pas }
  TRGBQuadArray = array [Word] of TRGBQuad;
  PRGBQuadArray = ^TRGBQuadArray;

  { TImageBrightness }
  TImageBrightness = -255..255;

  { color and image manipulation functions }
  function MakeBrightColor(const Original : TColor; const Brightness : TImageBrightness) : TColor;
  procedure SetImageBrightness(const Bitmap : TBitmap; const Brightness : TImageBrightness);
  function  MakeSoftColor(const Original : TColor; const Softness : Byte) : TColor;
  procedure SetImageSoftness(const Bitmap : TBitmap; const Softness : Byte);
  function MakeGrayScaleColor(const Original : TColor) : TColor;
  procedure SetImageGrayScale(const Bitmap : TBitmap);
  function MakeGatedColor(const Original, Floor, Ceeling : TColor) : TColor;
  procedure SetImageGated(const Bitmap : TBitmap; const Floor, Ceeling : TColor);
  procedure SetImageAlternatingTransparency(const Bitmap : TBitmap; const Transparency : TColor);

  { image processing functions }
  procedure GetBitmapLinearHistogram(const Bitmap : TBitmap; PointA, PointB : TPoint; var Count : Integer; var QuadArray : TRGBQuadArray);

  { io and files routines }
  function OYSetFileNameExtentionIfNone(const FileName : String; const NameExt : String) : String;
  function OYGetFileNameExtention(const FileName : String) : String;
  function OYCompareFileModifiedTimestamp(FileName1, FileName2 : String) : Integer;
  function OYCanFileNamePassFilter(const FileName, Filter : String) : Boolean;
  procedure OYEnumFilesInFolders(const Path, FileFilter : String; const IncludeSubfolders : Boolean; FileList : TStrings);

  { vcl }
  procedure OYForceDoubleBufferedForControlTree(AControl : TWinControl; AClass : TClass; Value : Boolean);
  procedure OYForceFullRepaintForControlTree(AControl : TWinControl; Value : Boolean);

  { WNet }
  function OYGetMachineName(Default : String) : String;
  function OYGetUserName(Default : String) : String;

  { internet }
  function OYFetchContentOfWellFormedUrl(const URL : String) : String;
  procedure OYOpenUrlInDefaultBrowser(Url : String);

type
  { sorting and searching routines }
  TCompareMethod = function (Item1, Item2 : Pointer): Integer;
  TMatchMethod = function (Item : Pointer; const Value) : Integer;
  procedure OYQuickSort(List: TList; L, R : Integer; CompareMethod : TCompareMethod);
  function OYFind(const Value; List : TList; MatchMethod : TMatchMethod; var Index : Integer): Boolean;
  function OYFindFirst(const Value; SortedList : TList; MatchMethod : TMatchMethod; var Index : Integer): Boolean;
  function OYFindLast(const Value; SortedList : TList; MatchMethod : TMatchMethod; var Index : Integer): Boolean;

  { generic text and data comparison methods }
  function CompareText(S1, S2 : String) : Integer;
  function CompareInt(Int1, Int2 : Integer) : Integer;
  function CompareDWord(Int1, Int2 : DWord) : Integer;

  { menu functions }
  procedure FillPopUpMenuFromStrings(PopUpMenu : TPopUpMenu; PopUpMenuOnClick : TNotifyEvent; Strings : TStrings);

  { string manipulation operation }
  function OYGetUpToNextToken(Token : String; var s : String) : String;
  function OYGetUpToAndNextToken(BeginToken, EndToken : String; var s : String; out TokenValue : String) : String;

  { registry }
  procedure OYFillInStreamFromRegistry(RootKey : DWord; KeyName, Name : String; Stream : TStream);
  procedure OYFillInRegistryFromStream(RootKey : DWord; KeyName, Name : String; Stream : TStream);
  procedure OYDeleteKeyValue(RootKey : DWord; KeyName, Name : String);

type
  { character filter }
  TOYCharFilter = set of Byte;

  { javascript encoding/decoding routines }
  function OYJavaScriptEncodeValue(Value : String) : String;
  function OYJavaScriptDecodeValue(Value : String) : String;

  { Xml encoding/decoding routines }
  function OYAllCharsInRange(Value : String; PassTroughChars : TOYCharFilter) : Boolean;
  function OYXmlEncodeValue(Value : String) : String;
  function OYXmlEncodeName(Name : String) : String;
  function OYXmlDecodeString(Value : String) : String;

  { misc }
  function OYCreateGuid : String;
  procedure OYSetByteBit(var AByte : Byte; const BitIndex : Byte; const BitValue : Boolean);
  function OYGetByteBit(var AByte : Byte; const BitIndex : Byte) : Boolean;


const
  { allowed character ranges }
  cValidXmlNameChars : TOYCharFilter = [Ord('.'), Ord(':'), Ord('-'), 48..57, 65..126];
  cValidXmlValueChars : TOYCharFilter = [48..57, 65..126];
  cValidJavaScriptStringChars : TOYCharFilter = [32..33, 35..38, 40..126];

type
  { improved VCL classes }
  TOYObjectList = class(TList)
  public
    procedure Clear; override;
  end;

  { EOYException }
  EOYException = class(Exception)
  protected
    fNExt, fPrev : Exception;  
  public
    constructor Create(Msg : String; PreviousException : Exception);
    function NextException : Exception;
    function PrevException : Exception;
  end;

implementation

uses {vcl } ActiveX, ShellApi, WinInet, Registry, ComObj;

procedure FillPopUpMenuFromStrings(PopUpMenu:TPopUpMenu; PopUpMenuOnClick: TNotifyEvent; Strings: TStrings);
var
  i: Integer;
  MenuItem: TMenuItem;
begin
  PopUpMenu.Items.Clear;
  for i := 0 to Strings.Count - 1 do begin
    MenuItem         := TMenuItem.Create( PopUpMenu );
    MenuItem.Caption :=          Strings.Strings[ i ];
    MenuItem.Tag     := LongInt( Strings.Objects[ i ] );
    MenuItem.OnClick := PopUpMenuOnClick;
    PopUpMenu.Items.Add( MenuItem );
  end;
end;

function OYSetFileNameExtentionIfNone(const FileName : String; const NameExt : String) : String;
begin
  if OYGetFileNameExtention(FileName) = '' then begin
    Result := FileName + '.' + NameExt;
  end else begin
    Result := FileName;
  end;
end;

function OYGetFileNameExtention(const FileName : String) : String;
var
  aPos : Integer;
begin
  Result := ExtractFileName(FileName);
  aPos := LastDelimiter('.', Result);
  if (aPos > 0) then begin
    Result := Copy(Result, aPos + 1, Length(Result));
  end else begin
    Result := '';
  end;
  Result := UpperCase(Result);
end;


function OYCompareFileModifiedTimestamp(FileName1, FileName2 : String) : Integer;
var
  aSourceC, aSourceA, aSourceM, aMetaC, aMetaA, aMetaM : FileTime;
  aStream : TFileStream;
begin
  aStream := TFileStream.Create(FileName1, fmOpenRead or fmShareDenyWrite);
  try
    if not GetFileTime(aStream.Handle, @aSourceC, @aSourceA, @aSourceM) then begin
      RaiseLastWin32Error;
    end;
  finally
    aStream.Free;
  end;

  aStream := TFileStream.Create(FileName2, fmOpenRead or fmShareDenyWrite);
  try
    if not GetFileTime(aStream.Handle, @aMetaC, @aMetaA, @aMetaM) then begin
      RaiseLastWin32Error;
    end;
  finally
    aStream.Free;
  end;

  Result := CompareDWord(aSourceM.dwHighDateTime, aMetaM.dwHighDateTime);
  if (Result = 0) then begin
    Result := CompareDWord(aSourceM.dwLowDateTime, aMetaM.dwLowDateTime);
  end;                    
end;


function OYCanFileNamePassFilter(const FileName, Filter : String) : Boolean;
var
  aFilter : String;
  aIdx : Integer;
  aMask : String;
begin

  {
    We need to handle these cases:
      Filter1:='Text files(*.txt)|*.TXT';
      Filter2:='Text files(*.txt)|*.TXT|Pascal files (*.pas)|*.PAS';
      Filter3:='Pascal files|*.PAS;*.DPK;*.DPR';
  }

  { get rid of '|' }
  aFilter := Filter;
  repeat
    aIdx := Pos('|', aFilter);
    if aIdx = 0 then break;
    aFilter := Copy(aFilter, aIdx + 1, Length(aFilter));
    aIdx := Pos('|', aFilter);
    if aIdx = 0 then begin
      aMask := aMask + aFilter + ';';
      break;
    end else begin
      aMask := aMask + Copy(aFilter, 1, aIdx - 1) + ';';
      aFilter := Copy(aFilter, aIdx + 1, Length(aFilter));
    end;
  until False;

  { now we have '*.PAS;*.TXT;*.DPR;' etc; check if file name fits }
  Result := False;
  repeat
    aIdx := LastDelimiter(';', aMask);
    if aIdx = 0 then break;
    aMask := Copy(aMask, 1, aIdx - 1);
    aIdx := LastDelimiter('.', aMask);
    if aIdx = 0 then break;
    aFilter := Copy(aMask, aIdx + 1, Length(aMask));
    if UpperCase(OYGetFileNameExtention(FileName)) = UpperCase(aFilter) then begin
      Result := True;
      break;
    end;
  until False;
end;

procedure OYQuickSort(List: TList; L, R: Integer; CompareMethod : TCompareMethod);
var
  I, J: Integer;
  P, T: Pointer;
begin
  repeat
    I := L;
    J := R;
    P := List[(L + R) div 2];
    repeat
      while CompareMethod(List[I], P) < 0 do Inc(I);
      while CompareMethod(List[J], P) > 0 do Dec(J);
      if I <= J then begin
        T := List[I];
        List[I] := List[J];
        List[J] := T;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then OYQuickSort(List, L, J, CompareMethod);
    L := I;
  until I >= R;
end;

function OYFind(const Value; List : TList; MatchMethod : TMatchMethod; var Index: Integer): Boolean;
var
  L, H, I, C: Integer;
begin
  { find match }
  Result := False;
  L := 0;
  H := List.Count - 1;
  while L <= H do begin
    I := (L + H) shr 1;
    C := MatchMethod(List[I], Value);
    if C < 0 then begin
      L := I + 1;
    end else begin
      H := I - 1;
      if C = 0 then begin
        Result := True;
      end;
    end;
  end;
  Index := L;
end;

function OYFindFirst(const Value; SortedList : TList; MatchMethod : TMatchMethod; var Index: Integer): Boolean;
begin
  Result := OYFind(Value, SortedList, MatchMethod, Index);
  if Result then begin
    while Index > 0 do begin
      if MatchMethod(SortedList[Index - 1], Value) = 0 then begin
        Dec(Index);
      end else begin
        break
      end;
    end;
  end;

end;

function OYFindLast(const Value; SortedList : TList; MatchMethod : TMatchMethod; var Index: Integer): Boolean;
begin
  Result := OYFind(Value, SortedList, MatchMethod, Index);
  if Result then begin
    while Index < SortedList.Count - 1 do begin
      if MatchMethod(SortedList[Index + 1], Value) = 0 then begin
        Inc(Index);
      end else begin
        break
      end;
    end;
  end;
end;

function CompareText(S1, S2 : String) : Integer;
begin
  Result := SysUtils.CompareText(S1, S2);
end;

function CompareDWord(Int1, Int2 : DWord) : Integer;
begin
  if Int1 = Int2 then begin
    Result := 0;
  end else begin
    if Int1 > Int2 then begin
      Result := 1;
    end else begin
      Result := -1;
    end;
  end;
end;

function CompareInt(Int1, Int2 : Integer) : Integer;
begin
  if Int1 = Int2 then begin
    Result := 0;
  end else begin
    if Int1 > Int2 then begin
      Result := 1;
    end else begin
      Result := -1;
    end;
  end;
end;

procedure GetBitmapLinearHistogram(const Bitmap : TBitmap; PointA, PointB : TPoint; var Count : Integer; var QuadArray : TRGBQuadArray);
var
   xinc, yinc, denom, t, e : Integer;
   vertlonger : Boolean;
   aux, dx, dy : Integer;
   x, y : Integer;
begin
     if (PointA.x >= 0) and
        (PointA.y >= 0) and
        (PointA.x < Bitmap.Width) and
        (PointA.y < Bitmap.Height) and
        (PointB.x >= 0) and
        (PointB.y >= 0) and
        (PointB.x < Bitmap.Width) and
        (PointB.y < Bitmap.Height) then begin
       { prepare deltas }
       dx := PointB.x - PointA.x;
       dy := PointB.y - PointA.y;
       vertlonger := False;

       xinc := 1;
       yinc := 1;
       if dx < 0 then begin xinc := -1; dx := -dx; end;
       if dy < 0 then begin yinc := -1; dy := -dy; end;
       if dy > dx then begin vertlonger := True; aux := dx; dx := dy; dy := aux; end;
       denom := 2 * dx;
       t := 2 * dy;
       e := -dx;
       x := PointA.x;
       y := PointA.y;
       Count := 0;

       { trace the path }
       while dx >= 0 do begin
         Dec(dx);
         QuadArray[count] := PRGBQuadArray(Bitmap.ScanLine [Round(y)])[Round(x)];
         Inc(count);
         Inc(e, t);
         if e > 0 then begin
           if (vertlonger) then begin
             Inc(x, xinc);
           end else begin
             Inc(y, yinc);
           end;
           e := e - denom;
         end;
         if vertlonger then begin
           Inc(y, yinc);
         end else begin
           Inc(x, xinc);
         end;
       end;
     end;
end;

function MakeBrightColor(const Original : TColor; const Brightness : TImageBrightness) : TColor;
var
  aQuad : TRGBQuad;
begin
  aQuad := TRGBQuad(Original);
  with aQuad do begin
    if Brightness <> 0 then begin
      if Brightness > 0 then begin
        if (r + Brightness > 255) then r := 255 else Inc(r, Brightness);
        if (g + Brightness > 255) then g := 255 else Inc(g, Brightness);
        if (b + Brightness > 255) then b := 255 else Inc(b, Brightness);
      end else begin
        if (r + Brightness < 0) then r := 0 else Inc(r, Brightness);
        if (g + Brightness < 0) then g := 0 else Inc(g, Brightness);
        if (b + Brightness < 0) then b := 0 else Inc(b, Brightness);
      end;
    end;
  end;
  Result := TColor(aQuad);
end;

procedure SetImageBrightness(const Bitmap : TBitmap; const Brightness : TImageBrightness);
var
  Row, Col: Integer;
  Line: PRGBQuadArray;
begin
  if Bitmap.PixelFormat <> pf32bit then begin
    raise Exception.Create('Must use 32 bit/pixel format.');
  end;
  for Row := 0 to Bitmap.Height - 1 do begin
    Line := Bitmap.ScanLine [Row];
    for Col := 0 to Bitmap.Width - 1 do begin
      with Line [Col] do begin
        Line [Col] := TRGBQuad(MakeBrightColor(TColor(Line [Col]), Brightness));
      end;
    end;
  end;
end;

function MakeSoftColor(const Original : TColor; const Softness : Byte) : TColor;
var
  aQuad : TRGBQuad;
  aMax, aLift, aMin : Byte;
begin
  aQuad := TRGBQuad(Original);
  with aQuad do begin

    { find max component }
    aMax := 0;
    if r > aMax then begin
      aMax := r;
    end;
    if g > aMax then begin
      aMax := g;
    end;
    if b > aMax then begin
      aMax := b;
    end;

    { find min component }
    aMin := 255;
    if r < aMin then begin
      aMin := r;
    end;
    if g < aMin then begin
      aMin := g;
    end;
    if b < aMin then begin
      aMin := b;
    end;

    { lift all components that are set to min }
    aLift := aMax * Softness div 255;
    if r = aMin then begin
      r := aLift;
    end;
    if g = aMin then begin
      g := aLift;
    end;
    if b = aMin then begin
      b := aLift;
    end;

  end;
  Result := TColor(aQuad);
end;

procedure SetImageSoftness(const Bitmap : TBitmap; const Softness : Byte);
var
  Row, Col: Integer;
  Line: PRGBQuadArray;
begin
  if Bitmap.PixelFormat <> pf32bit then begin
    raise Exception.Create('Must use 32 bit/pixel format.');
  end;
  for Row := 0 to Bitmap.Height - 1 do begin
   Line := Bitmap.ScanLine [Row];
   for Col := 0 to Bitmap.Width - 1 do begin
     with Line [Col] do begin
       Line [Col] := TRGBQuad(MakeSoftColor(TColor(Line [Col]), Softness));
     end;
   end;
  end;
end;

function MakeGrayScaleColor(const Original : TColor) : TColor;
var
  aQuad : TRGBQuad;
  aGray : Byte;
begin
  aQuad := TRGBQuad(Original);
  with aQuad do begin
    aGray := (r + g + b) div 3;
    r := aGray;
    b := aGray;
    g := aGray;
  end;
  Result := TColor(aQuad);
end;

procedure SetImageGrayScale(const Bitmap : TBitmap);
var
  Row, Col: Integer;
  Line: PRGBQuadArray;
begin
  if Bitmap.PixelFormat <> pf32bit then begin
    raise Exception.Create('Must use 32 bit/pixel format.');
  end;
  for Row := 0 to Bitmap.Height - 1 do begin
    Line := Bitmap.ScanLine [Row];
    for Col := 0 to Bitmap.Width - 1 do begin
      with Line [Col] do begin
        Line [Col] := TRGBQuad(MakeGrayScaleColor(TColor(Line [Col])));
      end;
    end;
  end;
end;


procedure SetImageAlternatingTransparency(const Bitmap : TBitmap; const Transparency : TColor);
var
  Row, Col: Integer;
  Line: PRGBQuadArray;
begin
  if Bitmap.PixelFormat <> pf32bit then begin
    raise Exception.Create('Must use 32 bit/pixel format.');
  end;
  for Row := 0 to Bitmap.Height - 1 do begin
    Line := Bitmap.ScanLine [Row];
    for Col := 0 to Bitmap.Width - 1 do begin
      with Line [Col] do begin
        if ((Row + Col) mod 2 = 0) then begin
          Line [Col] := TRGBQuad(Transparency);
        end;
      end;
    end;
  end;
end;


function MakeGatedColor(const Original, Floor, Ceeling : TColor) : TColor;
var
  aQuad, aFloor, aCeeling : TRGBQuad;
begin
  aQuad := TRGBQuad(ColorToRgb(Original));
  aFloor := TRGBQuad(ColorToRgb(Floor));
  aCeeling := TRGBQuad(ColorToRgb(Ceeling));

  if aQuad.r < (aCeeling.r + aFloor.r) div 2 then aQuad.r := aFloor.r else aQuad.r := aCeeling.r;
  if aQuad.g < (aCeeling.g + aFloor.g) div 2 then aQuad.g := aFloor.g else aQuad.g := aCeeling.g;
  if aQuad.b < (aCeeling.b + aFloor.b) div 2 then aQuad.b := aFloor.b else aQuad.b := aCeeling.b;

  Result := TColor(aQuad);         
end;

procedure SetImageGated(const Bitmap : TBitmap; const Floor, Ceeling : TColor);
var
  Row, Col: Integer;
  Line: PRGBQuadArray;
begin
  if Bitmap.PixelFormat <> pf32bit then begin
    raise Exception.Create('Must use 32 bit/pixel format.');
  end;
  for Row := 0 to Bitmap.Height - 1 do begin
    Line := Bitmap.ScanLine [Row];
    for Col := 0 to Bitmap.Width - 1 do begin
      with Line [Col] do begin
        Line [Col] := TRGBQuad(MakeGatedColor(TColor(Line[Col]), Floor, Ceeling));
      end;
    end;
  end;
end;


function OYGetUpToNextToken(Token : String; var s : String) : String;
var
   p : Integer;
begin
     { This procedure extract substring from the string.  It extracts substring from the begining
       of the line to the place where token is found.  Return value is a substring.
       The token itself is cut out and clipped string is placed back in the s variable.  Function is case
       sensitive. }

     p := Pos(token, s);
     if (p > 0) then begin
          Result := Copy(s, 1, p - 1);
          s := Copy(s, p + length(token), length(s));
          end
     else
         Result := '';
end;

function OYGetUpToAndNextToken(BeginToken, EndToken : String; var s : String; out TokenValue : String) : String;
begin
     { This procedure extract substring from the string.  It extracts substring from the begining
       of the line to the place where token is found.  Return value is a substring.
       The token itself is cut out and returned as TokenValue parameter and clipped string is
       placed back in the s variable. Function is case sensitive. }

     Result := OYGetUpToNextToken(BeginToken, s);  
     TokenValue := OYGetUpToNextToken(EndToken, s);
end;

function CharEncode(Value : String; PassTroughChars : TOYCharFilter) : String;
var
  i : Integer;
begin
  {
    this routine converts normal strings into HTML/JS/Xml encoded strings
    'A' => '&#65;'
  }

  Result := '';
  for i := 1 to Length(Value) do begin
    if Ord(Value[i]) in PassTroughChars then begin
      Result := Result + Value[i];
    end else begin
      Result := Result + '&#' + IntToStr(Ord(Value[i])) + ';';
    end;
  end;
end;

function OYAllCharsInRange(Value : String; PassTroughChars : TOYCharFilter) : Boolean;
var
  i : Integer;
begin
  { this function checks if all characters are in the range }
  Result := True;
  for i := 1 to Length(Value) do begin
    if not (Ord(Value[i]) in PassTroughChars) then begin
      Result := False;
      break;
    end;
  end;
end;

function OYXmlEncodeName(Name : String) : String;
begin
  Result := CharEncode(Name, cValidXmlNameChars);
end;

function OYXmlEncodeValue(Value : String) : String;
begin
  Result := CharEncode(Value, cValidXmlValueChars);
end;

function OYXmlDecodeString(Value : String) : String;
begin
  Result := OYJavaScriptDecodeValue(Value);
end;                               

function OYJavaScriptEncodeValue(Value : String) : String;
begin
  Result := CharEncode(Value, cValidJavaScriptStringChars);
end;


function OYJavaScriptDecodeValue(Value : String) : String;
var
  i : integer;
  s, t : String;
begin
  {
    this routine converts JavaScript encoded strings into normal strings
    '&#65;' => 'A'
  }

  Result := '';
  i := 1;
  while i <= Length(Value) do begin
    if (i <= Length(Value) - 2) and (Value[i] = '&') then begin
      s := Copy(Value, i, Length(Value));
      OYGetUpToNextToken('&', s);
      t := OYGetUpToNextToken(';', s);
      if t = 'amp' then begin
        Result := Result + '&';
      end else begin
        OYGetUpToNextToken('#', t);
        if t <> '' then begin
          Result := Result + Chr(StrToInt(t));
        end;
      end;
      i := 1;
      Value := s;
      continue;
    end else begin
      Result := Result + Value[i];
    end;
    Inc(i);
  end;
end;

function OYGetMachineName(Default : String) : String;
var
  aBufferSize : DWord;
begin
  aBufferSize := MAX_COMPUTERNAME_LENGTH + 1;
  SetLength(Result, aBufferSize);
  if GetComputerName(@Result[1], aBufferSize) then begin
   SetLength(Result, aBufferSize);
  end else begin
   Result := Default;
  end;
end;

function OYGetUserName(Default : String) : String;
var
  aBufferSize : DWord;
begin
  aBufferSize := MAX_COMPUTERNAME_LENGTH + 1;
  SetLength(Result, aBufferSize);
  if GetUserName(@Result[1], aBufferSize) then begin
   SetLength(Result, aBufferSize);
   Result := PChar(Result);
  end else begin
   Result := Default;
  end;
end;


{

  needs clean up and need TRGBQuad put in;
  comes from ProteinManager color blending code in 3dPanel.pas


function GetBlendColor(Back, Forg : TColor; Width, Pos : Integer) : TColor;
type
  RGBType = array[0..3] of Byte;
var
  b, f : RGBType;
  sz : Single;
begin
  try
    f := RGBType(ColorToRGB(Forg));
    b := RGBType(ColorToRGB(Back));
    sz := Pos / Width;
    f[0] := b[0] - Round((b[0] - f[0]) * sz);
    f[1] := b[1] - Round((b[1] - f[1]) * sz);
    f[2] := b[2] - Round((b[2] - f[2]) * sz);
    Result := PALETTERGB(f[0], f[1], f[2]);
  except
    Result := Forg;
  end;

  if Pos >= Width then Result := Forg;
  if Pos <= 0 then Result := Back;
end;
}


function OYFetchContentOfWellFormedUrl(const URL : String) : String;
const
  cBufSize = 16 * 1024;
var
  hSession, hfile : hInternet;
  aBuf : String;
  aSz, aRead : DWord;
begin
  Result := '';
  hSession := InternetOpen('{F8A3814A-7E8F-4585-A605-20EE1BC8BEDA}', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  if Assigned(hSession) then begin
    hfile := InternetOpenUrl(hSession, PChar(URL), nil, 0, INTERNET_FLAG_RELOAD, 0);
    if Assigned(hFile) then begin
      repeat
        aSz := cBufSize;
        SetLength(aBuf, aSz);
        if not InternetReadfile(hFile, PChar(aBuf), aSz, aRead) or (aRead = 0) then begin
          break;
        end;
        SetLength(aBuf, aSz);
        Result := Result + aBuf;
      until False;
      InternetCloseHandle(hFile);
    end else begin
      RaiselastWin32Error;
    end;
    InternetCloseHandle(hSession);
  end else begin
    RaiselastWin32Error;
  end;
end;

procedure OYOpenUrlInDefaultBrowser(Url : String);
begin
  ShellExecute(0, 'open', PChar(Url), '', '', SW_SHOW);
end;


(*
<request header>

'0=POST'                                                       
'1=HTTP/1.1'
'2='
'3='
'4=/Tconfirmlogin'
'5=c:\inetpub\wwwroot\Tconfirmlogin'
'6=no-cache'
'7='
'8=image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/vnd.ms-powerpoint, application/vnd.ms-excel, application/msword, application/pdf, */*'
'9='
'10=localhost'
'11='
'12=http://localhost/pbw/cgi/fwcgi.exe/Tlogin'#$D#$A'13=Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; COM+ 1.0.2204)'
'14='
'15=application/x-www-form-urlencoded'
'16=50'
'17='
'18='
'19='
'20='
'21=127.0.0.1'
'22=127.0.0.1'
'23=/pbw/cgi/fwcgi.exe'
'24=80'
'25='
'26=Keep-Alive'
'27=pbwSessionId=1446641941893942469'
'28='

<request content>
'username=admin&userpass=admin&databaseid=809912720'



<responce header>
'Status: 200 OK'
'Set-Cookie: pbwSessionId=1446641941893942469; '
'Content-Type: text/html'
'Content-Length: 18137'
'Content:'#$D#$A#$D#$A


<responce header>

*)

procedure OYFillInStreamFromRegistry(RootKey : DWord; KeyName, Name : String; Stream : TStream);
var
  aR : TRegistry;
  aSz: Integer;
  aP : Pointer;
begin
  aR := TRegistry.Create;
  try
    aR.RootKey := RootKey;
    if not aR.OpenKey(KeyName, True) then begin
      raise Exception.Create('Unable to access registry key (' + KeyName + ').');
    end;
    if not aR.ValueExists(Name) then begin
      raise Exception.Create('Value does not exists (' + Name + ').');
    end;                                                       
    aSz := aR.GetDataSize(Name);
    GetMem(aP, aSz);
    try
      aR.ReadBinaryData(Name, aP^, aSz);
      Stream.Write(aP^, aSz);
    finally
      FreeMem(aP);
    end;
  finally
    aR.Free;
  end;
end;

procedure OYFillInRegistryFromStream(RootKey : DWord; KeyName, Name : String; Stream : TStream);
var
  aR : TRegistry;
  aSz: Integer;
  aP : Pointer;
begin
  aR := TRegistry.Create;
  try
    aR.RootKey := RootKey;
    if not aR.OpenKey(KeyName, True) then begin
      raise Exception.Create('Unable to access registry key (' + KeyName + ').');
    end;
    aSz := Stream.Size;
    GetMem(aP, aSz);
    try
      Stream.Read(aP^, aSz);
      aR.WriteBinaryData(Name, aP^, aSz);
    finally
      FreeMem(aP);
    end;
  finally
    aR.Free;
  end;
end;

procedure OYDeleteKeyValue(RootKey : DWord; KeyName, Name : String);
var
  aR : TRegistry;
begin
  aR := TRegistry.Create;
  try
    aR.RootKey := RootKey;
    if not aR.OpenKey(KeyName, True) then begin
      raise Exception.Create('Unable to access registry key (' + KeyName + ').');
    end;
    if aR.ValueExists(Name) then begin     
      aR.DeleteValue(Name);
    end;                     
  finally
    aR.Free;
  end;
end;

                                            
procedure OYForceDoubleBufferedForControlTree(AControl : TWinControl; AClass : TClass; Value : Boolean);
var
  i : Integer;
begin
  for i := 0 to AControl.ControlCount - 1 do begin
    if (AControl.Controls[i] is TWinControl) and (AControl.Controls[i] is AClass) then begin
      TWinControl(AControl.Controls[i]).DoubleBuffered := Value;
    end;

    if AControl.Controls[i] is TWinControl then begin
      OYForceDoubleBufferedForControlTree(TWinControl(AControl.Controls[i]), AClass, Value);
    end;
  end;
end;

procedure OYForceFullRepaintForControlTree(AControl : TWinControl; Value : Boolean);
var
  i : Integer;
begin
  for i := 0 to AControl.ControlCount - 1 do begin
    if (AControl.Controls[i] is TPanel) then begin        
      TPanel(AControl.Controls[i]).FullRepaint := Value;  
    end;

    if AControl.Controls[i] is TWinControl then begin
      OYForceFullRepaintForControlTree(TWinControl(AControl.Controls[i]), Value);
    end;
  end;
end;


{ TOYObjectList }

procedure TOYObjectList.Clear;
var           
  i : Integer;
begin
  for i := 0 to Count - 1 do begin
    TObject(Self[i]).Free;
  end;

end;

{ EOYException }
constructor EOYException.Create(Msg : String; PreviousException : Exception);
begin
  inherited Create(Msg);
  fPrev := PreviousException;
  fNext := nil;                                                                 
  if Exception.InheritsFrom(EOYException) or (Exception.ClassName = EOYException.ClassName) then begin
    EOYException(Exception).fNext := Self;
  end;
end;

function EOYException.NextException : Exception;
begin
  Result := fNext;
end;

function EOYException.PrevException : Exception;
begin
  Result := fPrev;
end;

procedure OYEnumFilesInFolders(const Path, FileFilter : String; const IncludeSubfolders : Boolean; FileList : TStrings);

  procedure EnumFiles(Directory: String);
  var
    Rec         : TSearchRec;
    Found       : integer;
    aFileName       : String;
  begin
    Directory := IncludeTrailingBackslash(Directory);
    Found := FindFirst( Directory + '*.*', FAAnyfile, Rec );
    try
      while (Found = 0) do begin
        if (( Rec.Attr and FADirectory) = FADirectory) and
           (( Rec.Name <> '.') and (Rec.Name <> '..' )) and
           IncludeSubfolders then begin
          EnumFiles(Directory  + Rec.Name);
        end;
        aFileName := Directory + Rec.Name;
        if ((Rec.Attr and FADirectory) <> FADirectory) and
           OYCanFileNamePassFilter(aFileName, FileFilter) then begin
          FileList.Add(aFileName);
        end;

        Found := FindNext(Rec);
      end;
    finally
      FindClose(Rec);
    end;
  end;

begin
  EnumFiles(Path);
end;

function OYCreateGuid : String;
var
   aG : TGuid;
begin
  CoCreateGuid(aG);
  Result := GuidToString(aG)
end;


procedure OYSetByteBit(var AByte : Byte; const BitIndex : Byte; const BitValue : Boolean);
var
  aM : Byte;
begin
  if BitIndex > 7 then begin
    raise Exception.Create('Value for the bit index is not valid (' + IntToStr(BitIndex) + ').');
  end;
  if BitValue then begin
    aM := 1;
  end else begin
    aM := 0;
  end;
  AByte := AByte or (aM shl BitIndex);
end;

function OYGetByteBit(var AByte : Byte; const BitIndex : Byte) : Boolean;
var
  aM : Byte;
begin
  if BitIndex > 7 then begin
    raise Exception.Create('Value for the bit index is not valid (' + IntToStr(BitIndex) + ').');
  end;
  aM := 1;
  Result := (aByte and (aM shl BitIndex)) <> 0;
end;









(*
procedure TPbvBkBookForm.DocumentsGridRefresh;
var
  i,j: Integer;
  DocumentObj : TPbvOdDocumentObj;
  alSectionObj: TPbvOdSectionObj;
  FileName: Array[0..254] of Char;
  ExeDir  : Array[0..254] of Char;
  ExeName : Array[0..254] of Char;
  Bitmap: TBitmap;
  Icon: TIcon;
  bFocused: Boolean;
begin
  if not dbObj.IsObj( CurrentSection.ObjId) then
    begin
    ShowMessageBadObject( cPbvBkBk_Str_09 );
    UIContext.UIState := UIState_Contents; { refreshes Book }
    exit;
    end;

  { Grid heading }
  with DocumentsPanelLabel do
    Caption := GetName( CurrentSection.ObjId) + ' ' + cPbvBkBk_Str_06;

  { Clear grid }
  with DocumentsGrid do
    for i := 0 to ColCount-1 do
      for j := 0 to RowCount-1 do
        begin
        Objects[i,j].Free;
        Objects[i,j] := nil;
        Cells[i,j] := '';
        end;

  { Populate grid }
  alSectionObj := dbObj.CreateSectionObj( CurrentSection.ObjId, [ ODOMReadOnly, ODOMUserLevel]);
  try
    with DocumentsGrid do
      begin

      EnableButton( DocumentsPanel, cDocumentsUpBtn, False );

      RowCount := 1;

      i := -1;
      for j := 0 to alSectionObj.Documents.Count-1 do
        begin

        if not alSectionObj.Documents.IsObjByIndex(j) then
          continue;

        inc(i);
        if RowCount < i+1 then
          RowCount := RowCount + 1;

        DocumentObj := dbObj.CreateDocumentObj( alSectionObj.Documents.ObjIdByIndex[j], [ ODOMReadOnly, ODOMUserLevel]);
        if DocumentObj.Name = '' then
          Cells[1,i] := ' ' { needed: can't add object unless cell contains a string }
        else
          Cells[1,i] := DocumentObj.Name;

        Objects[1,i] := DocumentObj;

        Cells[0,i] := ' '; { needed }
        Objects[0,i] := nil;

        { Store icon (as bitmap, for resizeability) in col. 0 }
        StrPCopy( FileName, TPbvUtilFile.ExpandPathname( DocumentObj.FileSpec));
        if FindExecutable( FileName, ExeDir, ExeName) > 32 then
//KLM -- $FFFFFFFF was -1 but that is not an acceptable cardinal value
//
          if ExtractIcon( HInstance, ExeName, $FFFFFFFF ) > 0 then
            begin
            Icon := TIcon.Create;
            Icon.Handle:= ExtractIcon( HInstance, ExeName, 0);
            Bitmap := TBitmap.Create;
            with Bitmap do
              begin
              Height := cIconSideSize;
              Width  := cIconSideSize;
              Canvas.Draw( 0, 0, Icon);
              end;
            Objects[0,i] := Bitmap;
            Icon.Free;
            end;
        end;

      if i = -1 then
        begin
        bFocused := Focused;
        Enabled := False;
        EnableButton( DocumentsPanel, cDocumentsDownBtn, False );

        if bFocused then
          MoveFocus;
        end
      else
        begin
        Enabled := True;
        EnableButton( DocumentsPanel, cDocumentsDownBtn, RowCount > 1 );
        end;

      Row := 0;
      end;
      if DocumentsGrid.ColCount > 1 then begin
        DocumentsGrid.ColWidths[ 1 ] := DocumentsGrid.ClientWidth - 1 - DocumentsGrid.ColWidths[ 0 ] - 1;
      end;

  finally
    alSectionObj.Free;
  end;
end;


*)



end.

