{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}

unit OYXmlConsumer;

interface

uses { vcl } Classes,
     { oy  } OYPackage;

type
  { in memory tree }
  TOYXmlTransientTree = class(TInterfacedObject, IOYLinguineConsumer)
  private
    fStream : TMemoryStream;
  private
    procedure write(const Value : String);
    function getXML : String;
  protected
    procedure AddRaw(const Value : WideString);
    procedure ElementAtOnce(const Name : WideString; const Value : WideString);
    procedure BeginElement(const Name : WideString);
    procedure BeginElementWithAttr(const Name : WideString; const Attr, AttrValue : WideString);
    procedure BeginElementWithAttrs(const Name : WideString; const Attr1, AttrValue1 : WideString; const Attr2, AttrValue2 : WideString);
    procedure AddValue(const Value : WideString);
    procedure EndElement(const Name : WideString);
  public
    constructor Create;
    destructor Destroy;
    property Xml : String read getXml;
  end;

implementation

uses { vcl} SysUtils, Windows, Consts, Forms,
     { oy } OYUtility;


{ TOYXmlTransientTree }
constructor TOYXmlTransientTree.Create;
begin
  inherited Create;
  fStream := TMemoryStream.Create;
end;

destructor TOYXmlTransientTree.Destroy;
begin
  fStream.Free;
  inherited;
end;

procedure TOYXmlTransientTree.write(const Value : String);
begin
  fStream.Write(Value[1], Length(Value));
end;

function TOYXmlTransientTree.getXML : String;
var
        sz : Integer;
        p : Pointer;
        buf : String;
begin
  sz := fStream.Size;
  if sz = 0 then begin
    Result := '';
    exit;
  end;
  SetLength(buf, sz);
  fStream.Position := 0;
  fStream.Read(buf[1], sz);

  Result := buf;
end;

procedure TOYXmlTransientTree.ElementAtOnce(const Name : WideString; const Value : WideString);
begin
  BeginElement(Name);
  try
    AddValue(Value);
  finally
    EndElement(Name);
  end;
end;

procedure TOYXmlTransientTree.BeginElementWithAttr(const Name : WideString; const Attr, AttrValue : WideString);
begin
  write('<' + Name);
  if (Attr <> '') and (AttrValue <> '') then begin
    write(' ' + Attr + '="' + AttrValue + '"');
  end;
  write('>');
end;

procedure TOYXmlTransientTree.BeginElementWithAttrs(const Name : WideString; const Attr1, AttrValue1 : WideString; const Attr2, AttrValue2 : WideString);
begin
  write('<' + Name);
  if (Attr1 <> '') and (AttrValue1 <> '') then begin
    write(' ' + Attr1 + '="' + AttrValue1 + '"');
  end;
  if (Attr2 <> '') and (AttrValue2 <> '') then begin
    write(' ' + Attr2 + '="' + AttrValue2 + '"');
  end;
  write('>');
end;

procedure TOYXmlTransientTree.BeginElement(const Name : WideString);
begin
  BeginElementWithAttr(Name, '', '');
end;                                           

procedure TOYXmlTransientTree.AddRaw(const Value : WideString);   
begin
  write(Value);
end;

procedure TOYXmlTransientTree.AddValue(const Value : WideString);
begin
  write(OYXmlEncodeValue(Value));
end;

procedure TOYXmlTransientTree.EndElement(const Name : WideString);
begin
  write('</' + Name + '>'#$D#$A);
end;


end.


