{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYDelphi5DfmProducer;

interface

uses { vcl } SysUtils, Classes,
     { oy  } OYParserToken, OYYaccLib,
             OYApplication, OYPackage, OYProcessingInfoF, LexLib, OYDelphi5DfmLexer, OYDelphi5DfmParser;

type
  { TOYDelphi5DfmProducer }
  TOYDelphi5DfmProducer = class(TObject)
  private
    fIContext : IOYLinguineExecutionContext;
    fIConsumer : IOYLinguineConsumer;
    fIInfo : IOYLinguineProcessingInfo;
    procedure Warning(URL, reason, line, token : WideString; lineno, colno : Integer);
    procedure Hint(URL, reason, line, token : WideString; lineno, colno : Integer);
    function CanContinue : Boolean;
  public
    constructor Create(const IContext : IOYLinguineExecutionContext; const IConsumer : IOYLinguineConsumer);
    destructor Destroy; override;

  public
    function Execute(const Url : String) : Boolean;
    function GetProcessingInfo : IOYLinguineProcessingInfo;
  end;

  { TOYDelphi5DfmTemplatedParser }
  TOYDelphi5DfmTemplatedParser = class(TTemplatedParser)
  private
    fParsed : Boolean;
    fTokenIndex : Integer;
    fNeedOutput : Boolean;
    fIConsumer : IOYLinguineConsumer;
    fLexer : TLexer;
    fCurrent : Integer;
    fTokens : TList;
    fReady : Boolean;
  protected
    function Lex : TOYParserToken;
    function yylex : TOYParserToken; override;
    function CurrentToken : POYParserToken;
    procedure FirstToken;
    procedure LastToken;
    procedure NextToken;
    procedure PrevToken;
  public
    constructor Create(const Url : String; const IConsumer : IOYLinguineConsumer);
    destructor Destroy; override;
  public
    function yyparse : Boolean; override;
    procedure yyproduce(IConsumer : IOYLinguineConsumer);

    function lineByToken(Token : TOYParserToken) : String; override;
    function textByToken(Token : TOYParserToken) : String; override;
    function urlByToken(Token : TOYParserToken) : String; override;
  end;

  { TOYDelphi5DfmSchema }
  TOYDelphi5DfmSchema = class(TOYHighlightLinguineSchema)
  protected
    procedure ConfigureGroups; override;
  end;

  { TOYDelphi5DFMHighlighter }
  TOYDelphi5DfmHighlighter = class(TOYDelphi5SyntaxHighlighter)
  private
    fISchema : IOYLinguineSchema;
  protected    
    function IHighlightSchema : IOYLinguineSchema; override;
    function GetTokenIndexInSchemaForFrame(Frame : TOYLexerFrame) : Integer; override;
  end;

implementation

uses { oy } OYUtility;

{ TOYDelphi5DfmProducer }
constructor TOYDelphi5DfmProducer.Create(const IContext : IOYLinguineExecutionContext; const IConsumer : IOYLinguineConsumer);
begin
  inherited Create;
  fIContext := IContext;
  fIConsumer := IConsumer;
end;

destructor TOYDelphi5DfmProducer.Destroy;
begin
  fIContext := nil;
  fIConsumer := nil;
  inherited Destroy;
end;

procedure TOYDelphi5DfmProducer.Warning(URL, reason, line, token : WideString; lineno, colno : Integer);
begin
  fIInfo := TOYProcessingInfo.Create(icWarning, 1, Url, Reason, line, token, -1, Lineno, Colno, fIInfo);
end;

procedure TOYDelphi5DfmProducer.Hint(URL, reason, line, token : WideString; lineno, colno : Integer);
begin
  fIInfo := TOYProcessingInfo.Create(icHint, 1, Url, Reason, line, token, -1, Lineno, Colno, fIInfo);
end;

function TOYDelphi5DfmProducer.GetProcessingInfo : IOYLinguineProcessingInfo;
begin
  Result := fIInfo;
end;
        
function TOYDelphi5DfmProducer.CanContinue : Boolean;
begin
  Result := fIContext.IProgress.CanContinueBusy;
end;

function TOYDelphi5DfmProducer.Execute(const Url : String) : Boolean;
var
  aParser : TOYDelphi5DfmTemplatedParser;
begin
  try
    aParser := TOYDelphi5DfmTemplatedParser.Create(Url, fIConsumer);
    try
      aParser.OnWarning := Warning;
      aParser.OnHint := Hint;
      aParser.OnCanContinue := CanContinue;
      try                                                                       
        Result := aParser.yyparse;
        if not Result then begin
          fIInfo := TOYProcessingInfo.Create(icError, 1, aParser.urlByToken(aParser.currentToken^), 'Syntax error.', aParser.lineByToken(aParser.currentToken^), '', -1, aParser.currentToken.Frame.leftLineIndex, aParser.currentToken.Frame.leftColIndex, fIInfo);
        end;
      except
        on E : Exception do begin
          if aParser.currentToken = nil then begin
            fIInfo := TOYProcessingInfo.Create(icError, 1, Url, 'Unable to process binary file, text DFM format is required.', '', '', -1, -1, -1, fIInfo);
          end else begin
            fIInfo := TOYProcessingInfo.Create(icError, 1, aParser.urlByToken(aParser.currentToken^), E.Message, aParser.lineByToken(aParser.currentToken^), '', -1, aParser.currentToken.Frame.leftLineIndex, aParser.currentToken.Frame.leftColIndex, fIInfo);
          end;
          Result := False;
        end;
      end;
      if Result then begin
        aParser.yyproduce(fIConsumer);
      end;
    finally
      aParser.Free;
    end;
  except
    on E : Exception do begin
      fIInfo := TOYProcessingInfo.Create(icError, 1, Url, E.Message, '', '', -1, -1, -1, fIInfo);
      Result := False;
    end;
  end;
end;

{ TOYDelphi5DfmTemplatedParser }
constructor TOYDelphi5DfmTemplatedParser.Create(const Url : String; const IConsumer : IOYLinguineConsumer);
begin
  inherited Create(Url);

  fIConsumer := IConsumer;

  fParsed := False;
  fReady := False;

  fLexer := TTemplatedLexer.Create(Url);
  fTokens := TList.Create;
end;

destructor TOYDelphi5DfmTemplatedParser.Destroy;
var
  i : Integer;
begin
  for i :=0 to fTokens.Count - 1 do begin
    Dispose(POYParserToken(fTokens[i]));
  end;
  fTokens.Free;
  fLexer.Free;
  inherited Destroy;
end;

function TOYDelphi5DfmTemplatedParser.yylex : TOYParserToken;

   procedure _PushToken(Token : TOYParserToken);
   begin
     if fNeedOutput then begin
       FIConsumer.BeginElementWithAttr(cTokenElementName, cElementIdAttr, IntToStr(Token.Frame.Id));
       try
         FIConsumer.AddValue(textByToken(Token));
       finally
         FIConsumer.EndElement(cTokenElementName);
       end;
     end;
   end;

   procedure _PushPendingJunk(var S :String);
   begin
     if fNeedOutput then begin
       if (S <> '') then begin
         FIConsumer.BeginElementWithAttr(cTokenElementName, cElementIdAttr, IntToStr(tkJunk));
         try
           FIConsumer.AddValue(S);
           S := '';
         finally
           FIConsumer.EndElement(cTokenElementName);
         end;
       end;
     end;
   end;

   function _ShouldSkipToken(Token : TOYParserToken; var S : String) : Boolean;
   begin
     Result := True;
     case Token.Frame.Id of
       tkUnknown : begin     
         if Assigned(OnHint) then begin
           if Length(textByToken(Token)) = 1 then begin
             OnHint(urlByToken(Token), 'Unknown token (ASCII code ' + IntToStr(Ord(textByToken(Token)[1])) + ', replaced with "?").', lineByToken(Token), TextByToken(Token), Token.Frame.leftLineIndex, Token.Frame.leftColIndex);
           end else begin
             OnHint(urlByToken(Token), 'Unknown token (text "' + textByToken(Token) + '", replaced with "?").', lineByToken(Token), TextByToken(Token), Token.Frame.leftLineIndex, Token.Frame.leftColIndex);
           end;
         end;
         if OYAllCharsInRange(textByToken(Token), cValidXmlValueChars) then begin
           S := S + textByToken(Token);
         end else begin
           if fNeedOutput then begin
             FIConsumer.BeginElementWithAttr(cTokenElementName, cElementIdAttr, IntToStr(Token.Frame.Id));
             try
               FIConsumer.AddValue('?');
             finally
               FIConsumer.EndElement(cTokenElementName);
             end;
           end;
         end;
       end;
       tkCRLF, tkSpace{, tkUnknown} : begin
         S := S + textByToken(Token);
       end;
       else Result := False;
     end;          
   end;

var
  S : String;
begin
  S := '';
  repeat
    Result := Lex;
  until (Result.Lexer = nil) or not _ShouldSkipToken(Result, S);

  _PushPendingJunk(S);
  if Result.Lexer <> nil then begin
    Result.Index := fTokenIndex;
    inc(fTokenIndex);
  end;
end;

function TOYDelphi5DfmTemplatedParser.Lex : TOYParserToken;
var
  aRes : Integer;
  pToken : POYParserToken;
  aToken : TOYParserToken;
begin
  if not fReady then begin
    repeat
      aRes := fLexer.lex;
      if aRes <> 0 then begin
        New(pToken);
        pToken^.Lexer := fLexer;
        pToken^.Frame := fLexer.frame;
        fTokens.Add(pToken);
        aToken := pToken^;
        Beep;
      end else begin
        break;
      end;
    until False;

    if fTokens.Count = 0 then begin
      raise Exception.Create('No tokens found.');
    end;

    fCurrent := 0;
    fReady := True;
  end;

  FillChar(Result, SizeOf(Result), 0);
  if (fTokens.Count > 0) and (fCurrent < fTokens.Count) then begin
    Result := POYParserToken(fTokens[fCurrent])^;
    NextToken;
  end;
end;

function TOYDelphi5DfmTemplatedParser.yyparse : Boolean;
begin
  fTokenIndex := 0;
  fNeedOutput := False;

  Result := inherited yyparse;
  fParsed := Result;
end;

procedure TOYDelphi5DfmTemplatedParser.yyproduce;
begin              
  if not fParsed then begin
    raise Exception.Create('Unable to produce output(parsing incomplete).');
  end;
  fTokenIndex := 0;
  fNeedOutput := True;

  TransformFrames;
  SortFrames;

  FirstToken;
  WriteToConsumer(fIConsumer);
end;

function TOYDelphi5DfmTemplatedParser.lineByToken(Token : TOYParserToken) : String;
begin
  Result := TLexer(Token.Lexer).lineByFrame(Token.Frame);
end;

function TOYDelphi5DfmTemplatedParser.textByToken(Token : TOYParserToken) : String;
begin
  Result := TLexer(Token.Lexer).textByFrame(Token.Frame);
end;

function TOYDelphi5DfmTemplatedParser.urlByToken(Token : TOYParserToken) : String;
begin
  Result := TLexer(Token.Lexer).url;
end;

function TOYDelphi5DfmTemplatedParser.CurrentToken : POYParserToken;
begin
  Result := nil;
  if (fTokens.Count > 0) and (fCurrent < fTokens.Count) then begin
    Result := fTokens[fCurrent];
  end;
end;

procedure TOYDelphi5DfmTemplatedParser.FirstToken;
begin
  fCurrent := 0;
end;

procedure TOYDelphi5DfmTemplatedParser.LastToken;
begin
  fCurrent := fTokens.Count;
end;

procedure TOYDelphi5DfmTemplatedParser.NextToken;
begin
  Inc(fCurrent);
  if fCurrent > fTokens.Count then begin
    fCurrent := fTokens.Count;
  end;
end;

procedure TOYDelphi5DfmTemplatedParser.PrevToken;
begin
  Dec(fCurrent);
  if fCurrent < 0 then begin
    fCurrent := 0;
  end;
end;

{ TOYDelphi5DFMHighlighter }
function TOYDelphi5DFMHighlighter.IHighlightSchema : IOYLinguineSchema;
begin
  if not Assigned(fISchema) then begin
    fISchema := TOYDelphi5DfmSchema.Create;
  end;
  Result := fISchema;
end;

function TOYDelphi5DFMHighlighter.GetTokenIndexInSchemaForFrame(Frame : TOYLexerFrame) : Integer;
begin
  Result := 0;
  case Frame.Id of
    tkObject, tkEnd,
    tkSemiColon, tkColon, tkEqual, tkMinus,
    tkGreater, tkLower,
    tkSquareOpen, tkSquareClose,
    tkRoundOpen, tkRoundClose : Result := 1;
    tkStringLiteral, tkAsciiChar : Result := 2;
    tkInteger, tkFloat : Result := 3;
    tkBinary : Result := 4;
  end;
end;

{ TOYDelphi5DfmSchema }
procedure TOYDelphi5DfmSchema.ConfigureGroups;
begin
  fGroups := 5;
  SetLength(fGroupNames, fGroups);
  fGroupNames[0].Name := 'Other';
  fGroupNames[0].Desc := 'All other language elements.';
  fGroupNames[1].Name := 'Reserved Words';
  fGroupNames[1].Desc := 'All reserved words.';
  fGroupNames[2].Name := 'String Literals';
  fGroupNames[2].Desc := 'All string literals.';
  fGroupNames[3].Name := 'Numbers';
  fGroupNames[3].Desc := 'All integer, float, and hexadecimal numbers.';
  fGroupNames[4].Name := 'Binary Data';
  fGroupNames[4].Desc := 'Binary data embedded in the form.';
end;


end.
