{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYApplication;

interface

type
  { generic progress interface }
  IOYProgress = interface(IUnknown)
    ['{48D6DE5E-86E6-0000-0000-000000000001}']
    procedure Abort;
    procedure BeginBusy(Sender : TObject);
    procedure EndBusy(Sender : TObject);
    function CanContinueBusy : Boolean;
    function IsBusy : Boolean;
    procedure SetProgress(Sender : TObject; Percentage : Integer);
    procedure SetMessage(Sender : TObject; Message : WideString);
    procedure SetStatus(Sender : TObject; Message : WideString);
    function DoesUserConfirmAction(Message : WideString; Default : Boolean) : Boolean;
  end;
  
  { all log events must be classified according to one of these categories }
  TOYEventCategory = (lcSevere, lcError, lcWarning, lcStatus, lcAudit, lcDebug, lcPerform);

  { general purpose logging interface }
  IOYLog = interface
    ['{48D6DE5E-86E6-0000-0000-000000000003}']
    procedure LogEvent(const Category : TOYEventCategory; const Event : WideString; const IData : IUnknown);
    procedure LogObjectEvent(const Sender : TObject; const Category : TOYEventCategory; const Event : WideString; const IData : IUnknown);
    procedure LogLogicalEvent(const Source : WideString; const Category : TOYEventCategory; const Event : WideString; const IData : IUnknown);
  end;
                          
  { interface list iterator }
  IOYLinguineIterator = interface
    ['{690356B6-2052-45FA-8C54-D1CBBF59D6C1}']
    function First : IUnknown;
    function Next(ICurrent : IUnknown) : IUnknown;
    function FindFirst(const IID: TGUID) : IUnknown;
    function FindNext(const IID: TGUID; ICurrent : IUnknown) : IUnknown;
  end;

  { registry of generic services }
  IOYLinguineRegistry = interface
    ['{48D6DE5E-86E6-0000-0000-000000000004}']
    procedure RegisterInterface(IInterface : IUnknown);
    procedure UnRegisterInterface(IInterface : IUnknown);
  end;

  { generic product identity }
  IOYLinguineIdentity = interface
    ['{48D6DE5E-86E6-0000-0000-000000000009}']
    function ExeNameNoExtention : WideString;    // 'linguine';
    function ExeNameWithExtention : WideString;  // '.exe';
    function MajorVersion : WideString;          // '1';
    function MinorVersion : WideString;          // '2';
    function Release : WideString;               // '0';
    function Build : WideString;                 // '861';
    function CompanyName : WideString;           // 'Outplay';
    function ProductName : WideString;           // 'codeLinguine 1.1';
    function ProductNameNoVersion : WideString;  // 'codeLinguine';
    function ProductGuid : WideString;           // '{EC449D26-B9EB-493C-AC35-33197BA2163A}';
    function ProductUrl : WideString;            // 'http://www.codeLinguine.com';
    function ProductVersionInfo : WideString;    // AppName + ' ' + MajorVersion + '.' + MinorVersion + '.' + Release + '.' + Build
    function ProductEdition : WideString;        // Global Edition
  end;

  { generic licensing services }
  IOYLinguineLicense = interface
    ['{48D6DE5E-86E6-0000-0000-000000000010}']
    function  LicenseName : WideString;          // 'Single User Lisense'
    function  LicenseGuid : WideString;          // '{81666537-DE60-46E6-9B05-9774DE9A49A9}';
    procedure AssertPackageLoad(const PackageGuid : WideString);
    procedure AssertPackageUnLoad(const PackageGuid : WideString);
    procedure AssertMaxWorkspaceSize(const Size : Integer);
    procedure AssertMaxFileSize(const Size : Integer);
    procedure AssertMaxInstances(const Instances : Integer);
    procedure AssertBatchSize(const Size : Integer);
    procedure AssertMaxPackageCount(const Count : Integer);
  end;

  { generic application options }
  IOYLinguineOptions = interface
    ['{48D6DE5E-86E6-0000-0000-000000000012}']
    function GetMaxLogFileSize : Integer;
  end;

  { generic execution context }
  IOYLinguineExecutionContext = interface(IUnknown)
    ['{F45AEAB9-2AE6-4819-ADD8-D186F98003BD}']
    function IProgress : IOYProgress;
  end;

  { generic application services }
  IOYLinguineApplication = interface
    ['{48D6DE5E-86E6-0000-0000-000000000006}']
    function IIdentity : IOYLinguineIdentity;
    function IOptions : IOYLinguineOptions;
    function ILog : IOYLog;
    function IServices : IOYLinguineIterator;
    function IRegistry : IOYLinguineRegistry;
    function IExecutionContext : IOYLinguineExecutionContext;
    function ILicense : IOYLinguineLicense;
    procedure Run;
    procedure Terminate;
    function Terminated : Boolean;
  end;
  
implementation

end.
