{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYPackage;

interface

uses { oy } OYApplication;  

type
  { forwards }
  IOYLinguineProcessingOptions = interface;
  IOYLinguineProcessingInfo = interface;
  IOYLinguineConsumer = interface;
  IOYSyntaxHighlight = interface;
  IOYLinguineLanguagePackage = interface;

  { generic processing options }
  IOYLinguineProcessingOptions = interface(IUnknown)
    ['{7F34260B-8B17-42F4-A39F-93A906C4D9C4}']
    function GetDefines : WideString;               // returns list of conditional defines separated by the CR/LF
    function GetSearchPaths : WideString;           // returns list of search paths separated by CR/LF
    function IgnoreWhitespace : Boolean;            // returns flag that controls white space processing 
  end; 

  { procesing event category }
  TOYLinguineProcessingInfoCategory = (icError, icWarning, icHint);   // hint/warning allow completion of parsing and output production; error does not

  { generic interface for reporting procesing status for methods
  that can have partial completion when completion is possible }
  IOYLinguineProcessingInfo = interface(IUnknown)
    ['{FECFFDCA-6A26-48DA-B377-259A81AA4C1D}']
    function GetCategory : TOYLinguineProcessingInfoCategory;         // gets error category
    function GetErrorCode: Integer;                                   // gets error code
    function GetUrl: WideString;                                      // gets url where error occured
    function GetReason: WideString;                                   // gets human readable reason for the error
    function GetSrcText: WideString;                                  // get test element that resulted in the error
    function GetPos: Integer;                                         // gets absolute file position for the error
    function GetLine : WideString;                                    // gets text line where error occured
    function GetLineNo : Integer;                                     // gets line number for the error 
    function GetLinePos: Integer;                                     // get line position for the error
    function GetFirstInfo : IOYLinguineProcessingInfo;                // gets first info
    function GetLastInfo : IOYLinguineProcessingInfo;                 // gets last info
    function GetNextInfo : IOYLinguineProcessingInfo;                 // gets previous info
    function GetPrevInfo : IOYLinguineProcessingInfo;                 // gets next info
    procedure AttachInfo(const IInfo : IOYLinguineProcessingInfo);    // attaches new info at the end of current one shifting item down in the chain
  end;

  { generic consumer of capturing hierarchical output }
  IOYLinguineConsumer = interface(IUnknown)
    ['{865FF28B-F2F6-45B7-8339-D71EC47BA7A1}']
    procedure AddRaw(const Value : WideString);                       // adds raw text without any encoding
    procedure ElementAtOnce(
      const Name : WideString; const Value : WideString               // begins/ends element with encoding
    );
    procedure BeginElement(const Name : WideString);                  // begins element with encoding
    procedure BeginElementWithAttr(
      const Name : WideString; const Attr, AttrValue : WideString     // begin element (with attributes) with encoding
    );
    procedure BeginElementWithAttrs(
      const Name : WideString;                                        // begin element (with attributes) with encoding
      const Attr1, AttrValue1 : WideString;
      const Attr2, AttrValue2 : WideString
    );
    procedure AddValue(const Value : WideString);                     // adds element contents with encoding
    procedure EndElement(const Name : WideString);                    // ends element with encoding 
  end;

  { generic schema interafce }
  IOYLinguineSchema = interface(IUnknown)
    ['{EA8A56E7-CA70-4650-BCA8-8C29C1940210}']
    function GetCount : Integer;                               // returns number of elements in schema
    function GetId(const Index : Integer) : Integer;           // returns id of the element
    function GetName(const Index : Integer) : WideString;      // gets name of the element
    function GetSemantics(const Index : Integer) : WideString; // gets semantics(description) of the element
  end;

  { generic syntax highlighter }
  IOYSyntaxHighlight = interface(IUnknown)
    ['{D31284D9-D905-4E7E-9A53-0CFBEA9A0A63}']
    function IHighlightSchema : IOYLinguineSchema;   // highlighter schema
    function GetTokenIndexInSchema(
      const LineNo, LinePos : Integer
    ) : Integer;                                     // returns token index for given line/pos
    function IPackage : IOYLinguineLanguagePackage;  // gets package that highlighter relies upon
  end;                                                                             

  { generic processing package interface }
  IOYLinguineLanguagePackage = interface(IUnknown)
    ['{EEF798AC-8154-400A-BEF8-343C00BBA1FA}']
    function PackageGuid : WideString;               // '{3F0E72F2-EAF6-4D10-9A74-E48BE8B95B18}';
    function PackageCopyright : WideString;          // (C) 2001 XYZ Inc,
    function BaseLanguageVersion : WideString;       // version of this package
    function BaseLanguage : WideString;              // Borland Object Pascal, C++, Java
    function BaseLanguageDescription : WideString;   // human readable description of package
    function FileNameFilter : WideString;            // PAS, C, CPP, JAVA, ...
    function ILexSchema : IOYLinguineSchema;         // lex schema
    function IYaccSchema : IOYLinguineSchema;        // yacc schema
    function CreateSyntaxHighlighterForUrl(
      const Url : WideString;
      const IContext : IOYLinguineExecutionContext;
      const IOptions : IOYLinguineProcessingOptions;
      out IHighlighter : IOYSyntaxHighlight;
      out IInfo : IOYLinguineProcessingInfo
    ) : Boolean;                                     // produces highlighter interface
    function ProduceFullXmlTreeForUrl(
      const Url : WideString;
      const IContext : IOYLinguineExecutionContext;
      const IOptions : IOYLinguineProcessingOptions;
      const IConsumer : IOYLinguineConsumer;
      out   IInfo : IOYLinguineProcessingInfo                                   
    ) : Boolean;                                     // compiles source to XML
  end;

type
  { this method type must have matching calling convention
    to what ever declared in DLL export section including "stdcall" }
  TOYPackageLoadingMethod = function (
      const IContext : IOYLinguineExecutionContext;
      out   IPackage : IOYLinguineLanguagePackage
  ) : HResult; stdcall;

const
  { all DLL's with package implementations must have this factory function }
  cOYPackageFactoryMethodName = 'OYCreatePackageInstance';

implementation

end.

