{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  

unit OYYaccLib;

interface

uses { vcl } Classes,
     { yl } YaccLib,
     { oy } OYApplication, OYPackage, OYParserToken, OYLexLib;

const
  { XML element names }
  cTokenElementName = 't';
  cRuleElementName = 'r';
  cElementIdAttr = 'id';
  cElementLineNumAttr = 'ln';
  cRootElementName = 'url';

type
  { TOYParser }
  TOYParser = class(TParser)
  private
    fFrameIndex : Integer;
  protected
    fFramesByLeft, fFramesByRight : TList;
    procedure LogFrameList(List : TList);
    procedure SortFrames;
    procedure TransformFrames;
    procedure WriteToConsumer(IConsumer : IOYLinguineConsumer);
    procedure yyframe(left, right : TOYParserToken; rule : Integer); override;
  public
    constructor Create(const Url : String);
    destructor Destroy; override;
  public
    function yyparse : Boolean; override;
  end;

  { TOYParserClass }
  TOYParserClass = class of TOYParser;

  { TOYYaccLinguineSchema }
  TOYYaccLinguineSchema = class(TInterfacedObject, IOYLinguineSchema)
  private
    fParserClass : TOYParserClass;
  protected
    function GetCount : Integer;
    function GetId(const Index : Integer) : Integer;
    function GetName(const Index : Integer) : WideString;
    function GetSemantics(const Index : Integer) : WideString;
  public
    constructor Create(ParserClass : TOYParserClass);
  end;

  { TOYLexLinguineSchema }
  TOYLexLinguineSchema = class(TInterfacedObject, IOYLinguineSchema)
  private
    fParserClass : TOYParserClass;
  protected
    function GetCount : Integer;
    function GetId(const Index : Integer) : Integer;
    function GetName(const Index : Integer) : WideString;
    function GetSemantics(const Index : Integer) : WideString;
  public
    constructor Create(ParserClass : TOYParserClass);
  end;

  { TOYSchemaData }
  TOYSchemaData = record
    Name : String;
    Desc : String;
  end;

  { TOYHighlightLinguineSchema }
  TOYHighlightLinguineSchema = class(TInterfacedObject, IOYLinguineSchema)
  protected
    fGroups : Integer;
    fGroupNames : array of TOYSchemaData;
    procedure ConfigureGroups; virtual; abstract;
  protected
    function GetCount : Integer;
    function GetId(const Index : Integer) : Integer;
    function GetName(const Index : Integer) : WideString;
    function GetSemantics(const Index : Integer) : WideString;
  public
    constructor Create;
    destructor Destroy; override;      
  end;


  { TOYDelphi5SyntaxHighlighter }
  TOYDelphi5SyntaxHighlighter = class(TInterfacedObject, IOYSyntaxHighlight)
  private
    fUrl : WideString;
    fIContext : IOYLinguineExecutionContext;
    fIOptions : IOYLinguineProcessingOptions;
    fIInfo : IOYLinguineProcessingInfo;
    fReady : Boolean;
    fIndex : Integer;
    fLexer : TOYLexer;
    fFrames : array of TOYLexerFrame;
    fIPackage : IOYLinguineLanguagePackage;
    procedure Warning(URL, reason, line, token : WideString; lineno, colno : Integer);
    procedure Hint(URL, reason, line, token : WideString; lineno, colno : Integer);
    function CanContinue : Boolean;
    procedure ProcessFile(LexerClass : TOYLexerClass);
    function FindTokenIndexInSchema(const LineNo, LinePos : Integer) : Integer;
  protected
    function IHighlightSchema : IOYLinguineSchema; virtual; abstract;
    function GetTokenIndexInSchemaForFrame(Frame : TOYLexerFrame) : Integer; virtual; abstract;
    function GetTokenIndexInSchema( const LineNo, LinePos : Integer) : Integer;
    function IPackage : IOYLinguineLanguagePackage;
  public
    constructor Create(
      const IPackage : IOYLinguineLanguagePackage;
      const Url : WideString;
      const IContext : IOYLinguineExecutionContext;
      const IOptions : IOYLinguineProcessingOptions;
      const LexerClass : TOYLexerClass
    );
    destructor Destroy; override;
  end;

  procedure OYYaccSchemaToHtml(const IPackage : IOYLinguineLanguagePackage; const FileName : String);

implementation

uses { vcl } SysUtils,
     { oy } OYProcessingInfoF, OYUtility, OYVerInfo;

function SortFramesByLeft(Item1, Item2: Pointer) : Integer;
begin
  if POYParserFrame(Item1)^.Left < POYParserFrame(Item2)^.Left then begin
    Result := -1;
  end else begin
    if POYParserFrame(Item1)^.Left > POYParserFrame(Item2)^.Left then begin
      Result := 1;
    end else begin
      { sort by frame index }
      if POYParserFrame(Item1)^.Index < POYParserFrame(Item2)^.Index then begin
        Result := -1;
      end else begin
        if POYParserFrame(Item1)^.Index > POYParserFrame(Item2)^.Index then begin
          Result := 1;
        end else begin
          Result := 0;
        end;
      end;
    end;
  end;
end;

function SortFramesByRight(Item1, Item2: Pointer) : Integer;
begin
  if POYParserFrame(Item1)^.Right < POYParserFrame(Item2)^.Right then begin
    Result := -1;
  end else begin
    if POYParserFrame(Item1)^.Right > POYParserFrame(Item2)^.Right then begin
      Result := 1;
    end else begin
      { sort by frame index }
      if POYParserFrame(Item1)^.Index < POYParserFrame(Item2)^.Index then begin
        Result := -1;
      end else begin
        if POYParserFrame(Item1)^.Index > POYParserFrame(Item2)^.Index then begin
          Result := 1;
        end else begin
          Result := 0;
        end;
      end;
    end;
  end;
end;

{ TOYParser }
constructor TOYParser.Create(const Url : String);
begin
  inherited Create(Url);

  fFramesByLeft := TList.Create;
  fFramesByRight := TList.Create;
end;

destructor TOYParser.Destroy;
var
  i : Integer;
begin
  for i := 0 to fFramesByLeft.Count - 1 do begin
    Dispose(POYParserFrame(fFramesByLeft[i]));
  end;

  fFramesByRight.Free;
  fFramesByLeft.Free;
  inherited Destroy;
end;

procedure TOYParser.SortFrames;
var
  i : Integer;
begin
  fFramesByRight.Capacity := fFramesByLeft.Count;

  for i := 0 to fFramesByLeft.Count - 1 do begin
    fFramesByRight.Add(fFramesByLeft[i]);
  end;

  { the comparison methods must ensure that frames are sorted
  according to left and right indexes; if equal - sorting must
  continue by frame index }
  OYQuickSort(fFramesByLeft, 0, fFramesByLeft.Count - 1, SortFramesByLeft);
  OYQuickSort(fFramesByRight, 0, fFramesByRight.Count - 1, SortFramesByRight);
end;

procedure TOYParser.TransformFrames;
var
  aClosedFrames : TList;
  i, j : Integer;
  pFrame, pChildFrame : POYParserFrame;
begin
  {

  Here is the source:
    unit Test;
    interface
    uses A,B;
    implementation
    end.

  Here is list of non-whitespace tokens:
    0=unit
    1=Test
    2=;
    3=interface
    4=uses
    5=A
    6=,
    7=B
    8=;
    9=implementation
    10=end
    11=.

  Here is list of rule completion frames followed by the rule id:
    [1, 1]53
    [0, 2]52
    [5, 5]20
    [5, 5]19
    [7, 7]20
    [5, 7]18
    [4, 8]17
    [4, 4]66
    [3, 4]54
    [9, 9]16
    [9, 9]68
    [9, 9]55
    [10, 10]56
    [0, 11]51
    [0, 0]2

  Here is the token enclosure frames derived from the frames:
    0----|
    1]-53|-52-----------------------|
    2----|                          |
    3-------------------------|     |
    4----------------|        |-54  |
    5]-20]-19----|   |        |     |
    6            |-18|-17]-66-|     |-51]-2
    7]-20--------|   |              |
    8----------------|              |
    9]-16]-68                       |
   10]-56                           |
   11-------------------------------|

   The thing is: frame [3, 4] is not actually around tokens 3 and 4, it is
   around all tokens that are enclosed by any frame that enclose 3 and 4.
   The frames must be applied in hierarchical order of creation to properly
   construct enclosures.  The code below transforms completion frames into
   token enclosure frames.

  }
  
  aClosedFrames := TList.Create;
  try
    for i := 0 to fFramesByLeft.Count - 1 do begin
      pFrame := fFramesByLeft[i];
      j := 0;
      while j < aClosedFrames.Count do begin
        pChildFrame := aClosedFrames[j];
        if ((pChildFrame^.Left < pFrame^.Left ) and (pChildFrame^.Right < pFrame^.Left)) or
           ((pChildFrame^.Left > pFrame^.Right) and (pChildFrame^.Right > pFrame^.Right)) then begin
          inc(j);
          continue;
        end;
        if (pChildFrame^.Left < pFrame^.Left) then begin
          pFrame^.Left := pChildFrame^.Left;
        end;
        if (pChildFrame^.Right > pFrame^.Right) then begin
          pFrame^.Right := pChildFrame^.Right;
        end;
        aClosedFrames.Delete(j);
        continue;
      end;
      aClosedFrames.Add(pFrame);
    end;
  finally
    aClosedFrames.Free;
  end;
end;

procedure TOYParser.yyframe(left, right : TOYParserToken; rule : Integer);
var
  pFrame : POYParserFrame;
begin
  if yyruleLen(rule) > 0 then begin
    New(pFrame);
    if Left.Lexer = nil then begin
      pFrame^.Left := -1;
    end else begin
      pFrame^.Left := Left.Index;
    end;
    if Right.Lexer = nil then begin
      pFrame^.Right := -1;
    end else begin
      pFrame^.Right := Right.Index;
    end;
    pFrame^.Rule := Rule;
    pFrame^.Index := fFrameIndex;
    Inc(fFrameIndex);
    fFramesByLeft.Insert(fFramesByLeft.Count, pFrame);
  end;
end;

function TOYParser.yyparse : Boolean;
begin
  fFrameIndex := 0;
  Result := inherited yyparse;
end;

procedure TOYParser.LogFrameList(List : TList);
var
  i : Integer;
  pFrame : POYParserFrame;
  l, r : String;
begin
  yyhint('Report Frames');
  for i := 0 to List.Count - 1 do begin
    pFrame := List[i];
    l := IntToStr(pFrame^.left);
    r := IntToStr(pFrame^.right);
    yyhint('[' + l + ', ' + r + ']' + IntToStr(pFrame^.Rule));
  end;
end;

procedure TOYParser.WriteToConsumer(IConsumer : IOYLinguineConsumer);
var
  token : TOYParserToken;
  pFrame : POYParserFrame;
  left, right : Integer;
begin
  IConsumer.BeginElementWithAttr(cRootElementName, cElementIdAttr, Url);
  try
    left := 0;
    right := fFramesByRight.Count - 1;

    repeat
      Token := yyLex;
      if Token.Lexer = nil then break;

      repeat
        inc(left);
        if not ((left >= 0) and (left < fFramesByLeft.Count)) then break;
        pFrame := fFramesByLeft[left];
        if pFrame^.Left > Token.Index then break;
      until False;

      dec(left);
      while (left >= 0) and (left < fFramesByLeft.Count) do begin
        pFrame := fFramesByLeft[left];
        if pFrame^.Left <> Token.Index then break;
        IConsumer.BeginElementWithAttrs(
          cRuleElementName,
          cElementIdAttr, IntToStr(pFrame^.Rule),
          cElementLineNumAttr, IntToStr(Token.Frame.leftLineIndex)
        );
        dec(left);
      end;

      IConsumer.BeginElementWithAttrs(
        cTokenElementName,
        cElementIdAttr, IntToStr(Token.Frame.Id),
        cElementLineNumAttr, IntToStr(Token.Frame.leftLineIndex)
      );
      try
        IConsumer.AddValue(textByToken(Token));
      finally
        IConsumer.EndElement(cTokenElementName);
      end;

      repeat
        dec(right);
        if not ((right >= 0) and (right < fFramesByRight.Count)) then break;
        pFrame := fFramesByRight[right];
        if pFrame^.Right < Token.Index then break;
      until False;

      inc(right);
      while (right >= 0) and (right < fFramesByRight.Count) do begin
        pFrame := fFramesByRight[right];
        if pFrame.Right <> Token.Index then break;
        IConsumer.EndElement(cRuleElementName);
        inc(right);
      end;

    until False;

  finally
    IConsumer.EndElement(cRootElementName);
  end;
end;

{ TOYYaccLinguineSchema }
constructor TOYYaccLinguineSchema.Create(ParserClass : TOYParserClass);
begin
  inherited Create;

  fParserClass := ParserClass;
end;

function TOYYaccLinguineSchema.GetCount : Integer;
begin
  Result := fParserClass.yyruleCount;
end;

function TOYYaccLinguineSchema.GetId(const Index : Integer) : Integer;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := Index + 1;
end;

function TOYYaccLinguineSchema.GetName(const Index : Integer) : WideString;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := fParserClass.yyruleName(Index + 1);
end;

function TOYYaccLinguineSchema.GetSemantics(const Index : Integer) : WideString;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := fParserClass.yyruleBody(Index + 1);
end;

{ TOYLexLinguineSchema }
constructor TOYLexLinguineSchema.Create(ParserClass : TOYParserClass);
begin
  inherited Create;

  fParserClass := ParserClass;
end;

function TOYLexLinguineSchema.GetCount : Integer;
begin
  Result := fParserClass.yytokenCount;
end;

function TOYLexLinguineSchema.GetId(const Index : Integer) : Integer;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := fParserClass.yytokenId(Index + 1);
end;

function TOYLexLinguineSchema.GetName(const Index : Integer) : WideString;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := fParserClass.yytokenName(Index + 1);
end;

function TOYLexLinguineSchema.GetSemantics(const Index : Integer) : WideString;
begin
  { rule list array has 1-based idex; compensate for that here }
  Result := fParserClass.yytokenBody(Index + 1);
end;



{ TOYHighlightLinguineSchema }
constructor TOYHighlightLinguineSchema.Create;
begin
  inherited Create;

  ConfigureGroups;
end;

destructor TOYHighlightLinguineSchema.Destroy;
begin
  inherited Destroy;
end;

function TOYHighlightLinguineSchema.GetCount : Integer;
begin
  Result := fGroups;
end;

function TOYHighlightLinguineSchema.GetId(const Index : Integer) : Integer;
begin         
  Result := Index;
end;

function TOYHighlightLinguineSchema.GetName(const Index : Integer) : WideString;
begin
  if (Index >= 0) and (Index < fGroups) then begin
    Result := fGroupNames[Index].Name;
  end else begin
    raise Exception.Create('Invalid group index(' + IntToStr(Index) + ').');
  end;
end;

function TOYHighlightLinguineSchema.GetSemantics(const Index : Integer) : WideString;
begin
  if (Index >= 0) and (Index < fGroups) then begin
    Result := fGroupNames[Index].Desc;
  end else begin
    raise Exception.Create('Invalid group index(' + IntToStr(Index) + ').');
  end;
end;


{ TOYDelphi5SyntaxHighlighter }
constructor TOYDelphi5SyntaxHighlighter.Create(
  const IPackage : IOYLinguineLanguagePackage;
  const Url : WideString;
  const IContext : IOYLinguineExecutionContext;
  const IOptions : IOYLinguineProcessingOptions;
  const LexerClass : TOYLexerClass
);
begin
  inherited Create;
  fIPackage := IPackage;
  fUrl := Url;
  fIContext := IContext;
  fIOptions := IOptions;
  fFrames := nil;

  fReady := False;
  if not fReady then begin
    ProcessFile(LexerClass);
    fReady := True;
  end;
end;

destructor TOYDelphi5SyntaxHighlighter.Destroy;
begin
  if Assigned(fLexer) then begin
    fLexer.Free;
  end;
  SetLength(fFrames, 0);
  fFrames := nil;                                                           
  fIContext := nil;
  inherited Destroy;
end;

procedure TOYDelphi5SyntaxHighlighter.Warning(URL, reason, line, token : WideString; lineno, colno : Integer);
begin
  fIInfo := TOYProcessingInfo.Create(icWarning, 1, Url, Reason, line, token, -1, Lineno, Colno, fIInfo);
end;

procedure TOYDelphi5SyntaxHighlighter.Hint(URL, reason, line, token : WideString; lineno, colno : Integer);
begin
  fIInfo := TOYProcessingInfo.Create(icHint, 1, Url, Reason, line, token, -1, Lineno, Colno, fIInfo);
end;

function TOYDelphi5SyntaxHighlighter.CanContinue : Boolean;
begin
  if Assigned(fIContext) and Assigned(fIContext.IProgress) then begin
    Result := fIContext.IProgress.CanContinueBusy;
  end else begin
    Result := True;
  end;

end;

procedure TOYDelphi5SyntaxHighlighter.ProcessFile(LexerClass : TOYLexerClass);
const
  cCapacity = 32 * 1024;
var
  aRes : Integer;
  aSz, aCount : Integer;
begin
  fLexer := LexerClass.Create(fUrl);
  fLexer.OnWarning := Warning;
  fLexer.OnHint := Hint;
  fLexer.OnCanContinue := CanContinue;
  fIndex := 0;
  fReady := True;

  { collect all tokens in the array }
  aSz := 0;
  aCount := 0;
  repeat
    aRes := fLexer.lex;
    if aRes <> 0 then begin
      if aCount >= aSz then begin
        aSz := aSz + cCapacity;
        SetLength(fFrames, aSz);
      end;
      fFrames[aCount] := fLexer.frame;
      Inc(aCount);
    end else begin
      break;
    end;
    if ((Length(fFrames) mod 100) = 0) and (not CanContinue) then begin
      raise Exception.Create('Processing aborted.');
    end;
  until False;
  SetLength(fFrames, aCount);

  if Length(fFrames) = 0 then begin
    raise Exception.Create('No tokens found.');
  end;
end;

function TOYDelphi5SyntaxHighlighter.FindTokenIndexInSchema(const LineNo, LinePos : Integer) : Integer;

  function _CharInToken(Frame : TOYLexerFrame) : Boolean;
  begin
    Result :=
      ((Frame.leftLineIndex < LineNo) or ((Frame.leftLineIndex = LineNo) and (Frame.leftColIndex <= LinePos)) ) and
      ((Frame.rightLineIndex > LineNo) or ((Frame.rightLineIndex = LineNo) and (Frame.rightColIndex >= LinePos)) );
  end;

  function _PositionCurrentToken : Boolean;
  var
    i : Integer;
   begin
    Result := False;
    if (fFrames[fIndex].leftLineIndex > LineNo) or
       ((fFrames[fIndex].leftLineIndex = LineNo) and (fFrames[fIndex].leftColIndex > LinePos) )  then begin
      for i := fIndex downto 0 do begin
        if _CharInToken(fFrames[i]) then begin
          fIndex := i;
          Result := True;
          break;
        end;
      end;
    end else begin
      for i := fIndex to Length(fFrames) -  1 do begin
        if _CharInToken(fFrames[i]) then begin
          fIndex := i;
          Result := True;
          break;
        end;
      end;
    end;
  end;

begin
  FillChar(Result, SizeOf(Result), 0);
  if _CharInToken(fFrames[fIndex]) or _PositionCurrentToken then begin
    Result := GetTokenIndexInSchemaForFrame(fFrames[fIndex]);
  end else begin
    Result := -1;
  end;
end;

function TOYDelphi5SyntaxHighlighter.GetTokenIndexInSchema( const LineNo, LinePos : Integer) : Integer;
begin
  Result := FindTokenIndexInSchema(LineNo, LinePos);
end;

function TOYDelphi5SyntaxHighlighter.IPackage : IOYLinguineLanguagePackage;
begin
  Result := fIPackage;
end;

procedure OYYaccSchemaToHtml(const IPackage : IOYLinguineLanguagePackage; const FileName : String);
var
  aL, aM : TStringList;
  i : Integer;
  aN : String;

  function Link(Text : String) : String;
  begin
    Result := '<a href="#' + Text + '">' + Text + '</a>';
  end;

  function Row(A, B : String) : String;
  begin
    if A <> '' then begin
      A := '<a name="' + A + '"></a>' + A;
    end;
    Result :=
      '<tr>' +
      '<td><font size="1" face="Courier">' + A + '</font></td>' +
      '<td><font size="1" face="Courier">' + B + '</font></td>' +
      '</tr>';
  end;

  function FormatSemantics(Semantics : String) : String;
  var
    aB : String;
  begin
    repeat
      Result := OYGetUpToNextToken(' ', Semantics);
      if Result = '' then begin
        Result := Semantics;
        Semantics := '';
      end;
      if aM.IndexOf(Result) = -1 then begin
        aB := aB + ' ' + Result;
      end else begin
        aB := aB + ' ' + '<a href="#' + Result + '">' + Result + '</a>';
      end;
    until (Result = '') or (Semantics = '');
    Result := aB;
  end;

  function Head(Text : String) : String;
  begin
    Result := '<tr><td colspan="2" bgcolor="#444488"><font size="2" face="Arial" color="#FFFFFF"><b>' + Text + '</b></font></td></tr>';
  end;

  function Foot : String;
  begin
    Result :=
      '<tr><td colspan="2"><hr></td></tr>' +
      '<tr><td colspan="2">&nbsp;</td></tr>';
  end;

begin
  aL := TStringList.Create;
  aM := TStringList.Create;
  try
    for i := 0 to  IPackage.IYaccSchema.GetCount - 1 do  begin
      aM.Add(IPackage.IYaccSchema.GetName(i));
    end;
    aM.Sorted := True;

    aL.Add('<html><head><title>' + IPackage.BaseLanguage + '</title></head><body>');
    aL.Add('<table cellspacing="0" cellpadding="4" border="0">');

    aL.Add(Head('&nbsp;<br><a href="' + OYProductUrl + '"><font color="#FFFFFF">' + OYProductName + '</font></a> ' + OYProductEdition  + '<br>&nbsp;'));
    aL.Add(Foot);
                                            
    aL.Add(Head('Package schema information.'));
    aL.Add(Row ('Product Id', OYProductGuid));
    aL.Add(Row ('Package Id', IPackage.PackageGuid));
    aL.Add(Row ('Base language', IPackage.BaseLanguage));
    aL.Add(Row ('Base language description', IPackage.BaseLanguageDescription));
    aL.Add(Row ('Schema timestamp', DateTimeToStr(Now)));
    aL.Add(Foot);

    aL.Add(Head('Lexical analyzer tokens.'));
    i := 0 ;
    while (i < IPackage.ILexSchema.GetCount) do begin
      aL.Add(
        Row(IPackage.ILexSchema.GetName(i), '')
      );
      Inc(i);
    end;
    aL.Add(Foot);

    aL.Add(Head('Grammatical analyzer rules.'));
    i := 0;
    while (i < IPackage.IYaccSchema.GetCount) do begin
      aN := IPackage.IYaccSchema.GetName(i);
      aL.Add(
        Row(aN, ': ' + FormatSemantics(IPackage.IYaccSchema.GetSemantics(i)))
      );
      Inc(i);
      while (i < IPackage.IYaccSchema.GetCount) and (aN = IPackage.IYaccSchema.GetName(i)) do begin
        aL.Add(
          Row('&nbsp;', '| ' + FormatSemantics(IPackage.IYaccSchema.GetSemantics(i)))
        );
        Inc(i);
      end;
      aL.Add(
        Row('&nbsp;', ';')
      );
      aL.Add(Row('', ''));
    end;
    aL.Add(Foot);
    aL.Add(Head('&nbsp;<br>' + IPackage.PackageCopyright + '<br>&nbsp;'));

    aL.Add('</table></body></html>');
    aL.SaveToFile(FileName);
  finally
    aM.Free;
    aL.Free;
  end;
end;

end.


