{$I oycl.inc}  

{
  Turbo Pascal Lex and Yacc Version 4.1
  Compiler generator toolset for Turbo/Borland/Free Pascal and Delphi

  Copyright (c) 1990-2000 by Albert Graef
                        <ag@muwiinfa.geschichte.uni-mainz.de>
  Copyright (c) 1996 by Berend de Boer
                        <berend@pobox.com>
  Copyright (c) 1998 by Michael Van Canneyt
                        <Michael.VanCanneyt@fys.kuleuven.ac.be>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

}

unit YaccLib;

interface

uses { oy } OYParserToken;

const
  yymaxdepth = 4 * 1024;

type
  YYSType = TOYParserToken;

type
  TParser = class(TObject)
  protected
    yyerrflag : Integer;
    yylval : YYSType;
    yyflag : ( yyfnone, yyfaccept, yyfabort, yyferror );

    yychar     : Integer; (* current lookahead character *)
    yynerrs    : Integer; (* current number of syntax errors reported by the parser *)
    yydebug    : Boolean; (* set to true to enable debugging output of parser *)

    FUrl : String;

    fOnWarning : TOYParseEvent;
    fOnHint : TOYParseEvent;
    fOnCanContinue : TOYParserCanContinue;

    yystate, yysp, yyn : Integer;
    yys : array [1..yymaxdepth] of Integer;
    yyv : array [1..yymaxdepth] of YYSType;
    yyval : YYSType;

  protected
    function yylex : TOYParserToken; virtual; abstract;
    procedure yyframe(left, right : TOYParserToken; rule : Integer); virtual; abstract;
    procedure yyaction ( yyruleno : Integer ); virtual; abstract;
    procedure yyclearin;
    procedure yyaccept;
    procedure yyabort;
    procedure yyerrlab;
    procedure yyerrok;

    procedure yywarning ( msg : String );
    procedure yyhint ( msg : String );
    function yycancontinue : Boolean;

  public
    constructor Create(const Url : String);
    destructor Destroy; override;

  public
    function yyparse : Boolean; virtual; abstract;

    class function yyruleLen(Index : Word) : Integer; virtual; abstract;
    class function yyruleId(Index : Word) : Integer; virtual; abstract;
    class function yyruleName(Index : Word) : String; virtual; abstract;
    class function yyruleBody(Index : Word) : String; virtual; abstract;
    class function yyruleCount : Word; virtual; abstract;

    class function yytokenId(Index : Word) : Integer; virtual; abstract;
    class function yytokenName(Index : Word) : String; virtual; abstract;
    class function yytokenBody(Index : Word) : String; virtual; abstract;
    class function yytokenCount : Word; virtual; abstract;

    function lineByToken(Token : TOYParserToken) : String; virtual; abstract;
    function textByToken(Token : TOYParserToken) : String; virtual; abstract;
    function urlByToken(Token : TOYParserToken) : String; virtual; abstract;

  public
    property Url : String read fUrl;
    property OnWarning : TOYParseEvent read fOnWarning write fOnWarning;
    property OnHint : TOYParseEvent read fOnHint write fOnHint;
    property OnCanContinue : TOYParserCanContinue read fOnCanContinue write fOnCanContinue;

  end;

implementation

{ TParser }
constructor TParser.Create(const Url : String);
begin
  inherited Create;

  FillChar(yyval, SizeOf(yyval), 0);
  FUrl := Url;
end;(*create*)

destructor TParser.Destroy;
begin
  inherited Destroy;
end;(*destroy*)

procedure TParser.yywarning ( msg : String );
begin
  if Assigned(fOnWarning) then begin
    if yyval.Lexer <> nil then begin
      fOnWarning(urlByToken(yyval), msg, lineByToken(yyval), textByToken(yyval), yyval.Frame.leftLineIndex, yyval.Frame.leftColIndex);
    end else begin
      fOnWarning(fUrl, msg, '', '', -1, -1);
    end;
  end;
end(*yywarning*);

procedure TParser.yyhint ( msg : String );
begin
  if Assigned(fOnHint) then begin
    if yyval.Lexer <> nil then begin
      fOnHint(urlByToken(yyval), msg, lineByToken(yyval), textByToken(yyval), yyval.Frame.leftLineIndex, yyval.Frame.leftColIndex);
    end else begin
      fOnHint(fUrl, msg, '', '', -1, -1);
    end;
  end;
end(*yyhint*);

function TParser.yycancontinue : Boolean;
begin
  Result := True;
  if Assigned(fOnCanContinue) then begin
    Result := fOnCanContinue;
  end;
end;

procedure TParser.yyclearin;
begin
  yychar := -1;
end(*yyclearin*);

procedure TParser.yyaccept;
begin
  yyflag := yyfaccept;
end(*yyaccept*);

procedure TParser.yyabort;
begin
  yyflag := yyfabort;
end(*yyabort*);

procedure TParser.yyerrlab;
begin
  yyflag := yyferror;
end(*yyerrlab*);

procedure TParser.yyerrok;
begin
  yyerrflag := 0;
end(*yyerrork*);

end(*YaccLib*).
