{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                                             }
{              Outplay Consulting Code Library                }
{                                                             }
{   NOTICE: THIS PROGRAM CONTAINS CONFIDENTIAL INFORMATION    }
{   WHICH IS THE PROPERTY OF OUTPLAY. THE CONTENTS MAY NOT    }
{   BE USED OR DISCLOSED WITHOUT EXPRESS WRITTEN PERMISSION   }
{   OF THE OWNER.                                             }
{                                                             }
{                     WWW.OUTPLAY.COM                         }
{                                                             }
{          Copyright (c) 1997,98,99, 00 Outplay               }
{                                                             }
{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

{
  Turbo Pascal Lex and Yacc Version 4.1
  Compiler generator toolset for Turbo/Borland/Free Pascal and Delphi

  Copyright (c) 1990-2000 by Albert Graef
                        <ag@muwiinfa.geschichte.uni-mainz.de>
  Copyright (c) 1996 by Berend de Boer
                        <berend@pobox.com>
  Copyright (c) 1998 by Michael Van Canneyt
                        <Michael.VanCanneyt@fys.kuleuven.ac.be>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

}

unit LexLib;

interface

uses { vcl } Classes, SysUtils;

const
  max_chars   = 64 * 1024;
  max_text    = 64 * 1024;
  max_matches =  1 * 1024;
  max_rules   =       256;

type
  TLexer = class(TObject)
  private
    fStateStack : TList;
    fDefines : TStrings;
    fDefineStack : TList;
    fIsDefined : Boolean;

  protected
    yystate    : Integer; (* current state of lexical analyzer *)
    yyactchar  : Char;    (* current character *)
    yylastchar : Char;    (* last matched character (#0 if none) *)
    yyrule     : Integer; (* matched rule *)
    yyrejected : Boolean; (* current match rejected? *)
    yydone     : Boolean; (* yylex return value set? *)
    yyretval   : Integer; (* yylex return value *)

    bufptr     : Integer;
    buf        : array [1..max_chars] of Char;

    yystext            : String;
    yysstate, yylstate : Integer;
    yymatches          : Integer;
    yystack            : array [1..max_matches] of Integer;
    yypos              : array [1..max_rules] of Integer;
    yysleng            : Byte;

    yyoutput           : TStream;     (* output file *)
    yyline             : String;      (* current input line *)
    yylineno, yycolno  : Integer;     (* current input position *)
    yytext             : String;      (* matched text (should be considered r/o) *)

    fUrl               : WideString;
    fInput             : TStrings;

  protected
    function get_char : Char;
    procedure unget_char ( c : Char );

    procedure yydefinename( name : String );
    procedure yyundefname( name : String );
    procedure yyifdef( name : String );
    procedure yyifndef( name : String );
    procedure yyelse;
    procedure yyendif;

    procedure put_char ( c : Char );
    procedure put_string ( s : String );

    procedure yystart ( state : Integer );
    procedure yyfatal ( msg : String );
    procedure yyecho;
    procedure yymore;
    procedure yyless ( n : Integer );
    procedure yyreject;
    procedure yyreturn ( n : Integer );
    procedure yyreturnc ( c : Char );
    function yywrap : Boolean;
    procedure yynew;
    procedure yyscan;
    procedure yymark ( n : Integer );
    procedure yymatch ( n : Integer );
    function yyfind ( var n : Integer ) : Boolean;
    function yydefault : Boolean;
    procedure yyclear;

    procedure yypush ( state : Integer );
    procedure yypop;

  public
    constructor Create(const url : WideString; const StartState : Integer; const yyoutput : TStream);
    destructor Destroy; override;

  public
    function lex : Integer; virtual;
    function url : WideString;
    function line : WideString;
    function lineno : Integer;
    function colno : Integer;
    function text : WideString;

  end;

implementation

const nl = #10;  (* newline character *)

constructor TLexer.Create(const url : WideString; const StartState : Integer; const yyoutput : TStream);
begin
  inherited Create;

  fUrl := url;
  self.yyoutput := yyoutput;

  fDefineStack := TList.Create;
  fStateStack := TList.Create;
  fDefines := TStringList.Create;
  fInput := TStringList.Create;
  fInput.LoadFromFile(URL);

  yyclear;
  yysstate := StartState;
  fIsDefined := True;
end;(*create*)

destructor TLexer.Destroy;
begin
  if fDefineStack.Count <> 0 then begin
    raise Exception.Create(fUrl + IntToStr(yylineno) + ':' + IntToStr(yycolno) + ' $IFDEF without $ENDIF.');
  end;

  fInput.Free;
  fStateStack.Free;
  fDefines.Free;
  fDefineStack.Free;
  inherited Destroy;
end;(*destroy*)

procedure TLexer.yyfatal ( msg : String );
begin
  raise Exception.Create(msg);
end(*fatal*);

function TLexer.get_char : Char;
var
  i : Integer;
begin
  if (bufptr=0) and (yylineno < fInput.Count) then
    begin
      yyline := fInput[yylineno];
      inc(yylineno);
      yycolno := 1;
      buf[1] := nl;
      for i := 1 to length(yyline) do
        buf[i+1] := yyline[length(yyline)-i+1];
      inc(bufptr, length(yyline)+1);
    end;
  if bufptr>0 then
    begin
      get_char := buf[bufptr];
      dec(bufptr);
      inc(yycolno);
    end
  else
    get_char := #0;
end(*get_char*);

procedure TLexer.unget_char ( c : Char );
begin
  if bufptr=max_chars then yyfatal('Input buffer overflow.');
  inc(bufptr);
  dec(yycolno);
  buf[bufptr] := c;
end(*unget_char*);

procedure TLexer.put_char ( c : Char );
begin
  if c=#0 then
    { ignore }
  else if c=nl then
    yyoutput.write(#$D#$A, 2)
  else
    yyoutput.write(c, 1)
end(*put_char*);

procedure TLexer.put_string ( s : String );
begin
  yyoutput.write(s[1], Length(s))
end;

procedure TLexer.yyecho;
var
  i : Integer;
begin
  for i := 1 to Length(yytext) do
    put_char(yytext[i])
end(*echo*);

procedure TLexer.yymore;
begin
  yystext := yytext;
end(*yymore*);

procedure TLexer.yyless ( n : Integer );
var i : Integer;
begin
  for i := Length(yytext) downto n+1 do
    unget_char(yytext[i]);
  SetLength(yytext, n);
end(*yyless*);

procedure TLexer.yyreject;
var i : Integer;
begin
  yyrejected := true;
  for i := Length(yytext)+1 to yysleng do
    yytext := yytext+get_char;
  dec(yymatches);
end(*reject*);

procedure TLexer.yyreturn ( n : Integer );
begin
  if (fIsDefined) or (n = 0) then begin
    yyretval := n;
    yydone := true;
  end;
end(*return*);

procedure TLexer.yyreturnc ( c : Char );
begin
  if (fIsDefined) or (c = #0) then begin
    yyretval := ord(c);
    yydone := true;
  end;
end(*returnc*);

procedure TLexer.yystart ( state : Integer );
begin
  yysstate := state;
end(*start*);                        

procedure TLexer.yypush ( state : Integer );
begin
  fStateStack.Insert(fStateStack.Count, Pointer(yysstate));
  yystart( state );
end(*start*);

procedure TLexer.yypop;
begin
  yystart(Integer(fStateStack[fStateStack.Count - 1]));
  fStateStack.Delete(fStateStack.Count - 1);
end(*start*);

function TLexer.yywrap : Boolean;
begin
  yywrap := true;
end(*yywrap*);

procedure TLexer.yynew;
begin                                      
  if yylastchar<>#0 then
    if yylastchar=nl then
      yylstate := 1
    else
      yylstate := 0;
  yystate := yysstate+yylstate;
  yytext  := yystext;
  yystext := '';
  yymatches := 0;
  yydone := false;
end(*yynew*);

procedure TLexer.yyscan;
begin
  if Length(yytext)>max_text then yyfatal('Line is too long.');
  yyactchar := get_char;
  yytext := yytext + yyactchar;
end(*yyscan*);

procedure TLexer.yymark ( n : Integer );
begin
  if n>max_rules then yyfatal('Too many rules.');
  yypos[n] := Length(yytext);
end(*yymark*);

procedure TLexer.yymatch ( n : Integer );
begin
  inc(yymatches);
  if yymatches>max_matches then yyfatal('Match stack overflow.');
  yystack[yymatches] := n;
end(*yymatch*);

function TLexer.yyfind ( var n : Integer ) : Boolean;
begin
  yyrejected := false;
  while (yymatches>0) and (yypos[yystack[yymatches]]=0) do
    dec(yymatches);
  if yymatches>0 then
    begin
      yysleng := Length(yytext);
      n       := yystack[yymatches];
      yyless(yypos[n]);
      yypos[n] := 0;
      if Length(yytext)>0 then
        yylastchar := yytext[Length(yytext)]
      else
        yylastchar := #0;
      yyfind := true;
    end
  else
    begin
      yyless(0);
      yylastchar := #0;
      yyfind := false;
    end
end(*yyfind*);

function TLexer.yydefault : Boolean;
begin
  yyrejected := false;
  yyactchar := get_char;
  if yyactchar<>#0 then
    begin
      put_char(yyactchar);
      yydefault := true;
    end
  else
    begin
      yylstate := 1;
      yydefault := false;
    end;
  yylastchar := yyactchar;
end(*yydefault*);

procedure TLexer.yyclear;
begin
  bufptr := 0;
  yysstate := 0;
  yylstate := 1;
  yylastchar := #0;
  yytext := '';
  yystext := '';
end(*yyclear*);

procedure TLexer.yydefinename( name : String );
var
  i : Integer;
begin
  name := UpperCase(name);
  i := fDefines.IndexOf(name);
  if i = -1 then begin
    fDefines.Add(name);
  end;
end;

procedure TLexer.yyundefname( name : String );
var
  i : Integer;
begin
  name := UpperCase(name);
  i := fDefines.IndexOf(name);
  if i <> -1 then begin
    fDefines.Delete(i);
  end;
end;

procedure TLexer.yyifdef( name : String );
begin
  fDefineStack.Insert(fDefineStack.Count, Pointer(fIsDefined));
  name := UpperCase(name);
  fIsDefined := fDefines.IndexOf(name) <> -1;
end;

procedure TLexer.yyifndef( name : String );
begin
  fDefineStack.Insert(fDefineStack.Count, Pointer(fIsDefined));
  name := UpperCase(name);
  fIsDefined := fDefines.IndexOf(name) = -1;
end;

procedure TLexer.yyelse;
begin
  if fDefineStack.Count = 0 then begin
    raise Exception.Create(fUrl + IntToStr(yylineno) + ':' + IntToStr(yycolno) + ' $ELSE without $IFDEF.');
  end;
  fIsDefined := not fIsDefined;
end;

procedure TLexer.yyendif;
begin
  if fDefineStack.Count = 0 then begin
    raise Exception.Create(fUrl + IntToStr(yylineno) + ':' + IntToStr(yycolno) + ' $ENDIF without $IFDEF.');
  end;
  fIsDefined := Boolean(fDefineStack[fDefineStack.Count - 1]);
  fDefineStack.Delete(fDefineStack.Count - 1);
end;

function TLexer.line : WideString;
begin
  Result := yyline;
end;

function TLexer.lineno : Integer;
begin
  Result := yylineno;
end;

function TLexer.colno : Integer;
begin
  Result := yycolno;
end;

function TLexer.text : WideString;
begin
  Result := yytext;
end;

function TLexer.lex : Integer;
begin
  { must be derived from }
  raise Exception.Create('Abstract error.');
end;(*lex*)

function TLexer.url : WideString;
begin
  Result := fUrl;
end;(*url*)

end(*LexLib*).
