{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}

unit OYProgressForm;

interface

uses { vcl } Windows, Classes, Controls, Forms, ExtCtrls, StdCtrls,
             ComCtrls, Buttons,
     {  oy } OYApplication, Menus, ActnList, ImgList;

type
  { TOYProgressForm }
  TOYProgressForm = class(TForm)
    Panel: TPanel;
    ProgressPanel: TPanel;
    MessagePanel: TPanel;
    ProgressMessage: TLabel;
    ProgressBarPanel: TPanel;
    ProgressBar: TProgressBar;
    CancelButtonPanel: TPanel;
    Button1: TButton;
    procedure CancelButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
  private
    fOnCancel : TNotifyEvent;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    property OnCancelClick : TNotifyEvent read fOnCancel write fOnCancel;
  end;

  { IOYHotSwapProgress }
  IOYHotSwapProgress = interface(IUnknown)
    ['{C621AE9D-1ADA-4FA0-B90B-4DE166EC081E}']
    function AttachWidget(Form : TOYProgressForm) : Boolean;
    function DetachWidget(Form : TOYProgressForm) : Boolean;
  end;

  { factory }

  function OYCreateInstance(const ILog : IOYLog) : IOYProgress;

implementation

{$R *.DFM}

uses { vcl } SysUtils, Dialogs;

var
  gMouseHook : HHook;
  gKeyboardHook : HHook;
  gIsBusy : Boolean;
  gIProgress : IOYProgress;
  gDoHook : TList;

type
  { interfaced form }                                                      
  TProgressForm = class(TInterfacedObject, IOYProgress, IOYHotSwapProgress)
  private
    FAborted : Boolean;
    fILog : IOYLog;
    FBusyCount : Integer;
    FForm : TOYProgressForm;

  private
    procedure ProcessMessages;
    procedure OnCancel(Sender : TObject);
    function HasUserInterface : Boolean;

  protected
    procedure Abort;
    procedure BeginBusy(Sender : TObject);
    procedure EndBusy(Sender : TObject);
    function CanContinueBusy : Boolean;
    function IsBusy : Boolean;
    procedure SetProgress(Sender : TObject; Percentage : Integer);
    procedure SetMessage(Sender : TObject; Message : WideString);
    procedure SetStatus(Sender : TObject; Message : WideString);
    function DoesUserConfirmAction(Message : WideString; Default : Boolean) : Boolean;

    function AttachWidget(Form : TOYProgressForm) : Boolean;
    function DetachWidget(Form : TOYProgressForm) : Boolean;
    procedure HookWindow(Handle : Hwnd);
    procedure UnHookWindow(Handle : Hwnd);

  public
    constructor Create(const ILog : IOYLog);
    destructor Destroy; override;

  end;

{ gloabls }
function OYCreateInstance(const ILog : IOYLog) : IOYProgress;
var
  aForm : TProgressForm;
begin
  if gIProgress = nil then begin
    aForm := TProgressForm.Create(ILog);
    gIProgress := aForm;
  end;
  Result := gIProgress;
end;

{ mouse and keyboard hooks }
function IsHooked(Handle : Hwnd) : Boolean;
var
  i : Integer;
begin
  Result := False;
  for i :=0 to gDoHook.Count - 1 do begin
    if Hwnd(gDoHook[i]) = Handle then begin
      Result := True;
      break;
    end;
  end;
end;

function MouseHook(Code : Integer; WParam : WPARAM; LParam : LPARAM) : LResult; stdcall;
begin
  { trap all events if we are busy }
  if Code < 0 then begin
    Result := CallNextHookEx( gMouseHook, Code, WParam, LParam );
  end else begin
    if (gIsBusy) and (IsHooked(TMouseHookStruct(Pointer(LParam)^).Hwnd)) then begin
      Result := 1;
    end else begin
      Result := CallNextHookEx( gMouseHook, Code, WParam, LParam );
    end;
  end;
end;

function KeyboardHook(Code : Integer; WParam : WPARAM; LParam : LPARAM) : LResult; stdcall;
begin
  { trap all events if we are busy }
  if Code < 0 then begin
    Result := CallNextHookEx( gKeyboardHook, Code, WParam, LParam );
  end else begin
    if (gIsBusy) then begin
      Result := 1;
    end else begin
      Result := CallNextHookEx( gKeyboardHook, Code, WParam, LParam );
    end;
  end;
end;

{ TProgressForm }
constructor TProgressForm.Create(const ILog : IOYLog);
begin
  inherited Create;

  fILog := ILog;

  FAborted := False;
  gIsBusy := False;
end;

destructor TProgressForm.Destroy;
begin
  DetachWidget(FForm);

  if fBusyCount <> 0 then begin
    if Assigned(fILog) then begin
      fILog.LogObjectEvent(Self, lcSevere, 'Busy pending.', nil);
    end;
  end;
  
  inherited Destroy;
end;

function TProgressForm.AttachWidget(Form : TOYProgressForm) : Boolean;
begin
  Result := False;
  if not HasUserInterface then begin
    Result := True;
    FForm := Form;
    FForm.ProgressMessage.Caption := '';
    FForm.ProgressBar.Min := 0;
    FForm.ProgressBar.Max := 100;
    FForm.ProgressBar.Position := 0;
    FForm.OnCancelClick := OnCancel;

    { set hooks }
    gMouseHook := SetWindowsHookEx(WH_MOUSE, MouseHook, HInstance, GetCurrentThreadId);
    gKeyboardHook := SetWindowsHookEx(WH_KEYBOARD, KeyboardHook, HInstance, GetCurrentThreadId);
  end;
end;

function TProgressForm.DetachWidget(Form : TOYProgressForm) : Boolean;
begin
  Result := False;
  if HasUserInterface and (FForm = Form) then begin
    { remove hooks }
    UnhookWindowsHookEx(gMouseHook);
    UnhookWindowsHookEx(gKeyboardHook);

    FForm := nil;
    Result := True;
  end;
end;

procedure TProgressForm.HookWindow(Handle : Hwnd);
begin
  gDoHook.Add(Pointer(Handle));
end;

procedure TProgressForm.UnHookWindow(Handle : Hwnd);
begin
  gDoHook.Extract(Pointer(Handle));
end;

function TProgressForm.HasUserInterface : Boolean;
begin
  Result := (FForm <> nil) and not (Application.Terminated);
end;

procedure TProgressForm.OnCancel(Sender : TObject);
begin
  Abort;
end;

procedure TProgressForm.BeginBusy(Sender : TObject);
begin
  if FBusyCount = 0 then begin
    gIsBusy := True;
    Screen.Cursor := crHourGlass;
    FAborted := False;
    if HasUserInterface then begin
      FForm.position := poMainFormCenter;
      FForm.ProgressBar.Visible := False;
      FForm.ProgressBar.Position := 0;
      FForm.ProgressMessage.Caption := 'Executing...';
      FForm.Show;
      ProcessMessages;
    end;
  end;
  Inc(fBusyCount);
end;

procedure TProgressForm.EndBusy(Sender : TObject);
begin
  Dec(FBusyCount);
  if FBusyCount = 0 then begin
    SetProgress(Sender, 100);
    if HasUserInterface then begin
      FForm.Hide;
    end;
    Application.Restore;
    FAborted := False;
    Screen.Cursor := crDefault;
    gIsBusy := False;
  end;
end;

procedure TProgressForm.Abort;
begin
  if gIsBusy and not fAborted then begin
    if Assigned(fILog) then begin
      fILog.LogObjectEvent(Self, lcAudit, 'Execution aborted.', nil);
    end;
  end;
  fAborted := True;
end;

function TProgressForm.CanContinueBusy : Boolean;
begin
  if HasUserInterface then begin
    ProcessMessages;
  end;
  Result := not fAborted;
end;

function TProgressForm.IsBusy : Boolean;
begin
  Result := gIsBusy;
end;

procedure TProgressForm.SetProgress(Sender : TObject; Percentage : Integer);
begin
  if HasUserInterface then begin
    FForm.ProgressBar.Position := Percentage;
    FForm.ProgressBar.Visible := True;
    ProcessMessages;
  end;
end;

procedure TProgressForm.SetMessage(Sender : TObject; Message : WideString);
begin
  if HasUserInterface then begin
    FForm.ProgressMessage.Caption := Message;
    ProcessMessages;
  end;
end;

procedure TProgressForm.SetStatus(Sender : TObject; Message : WideString);
begin
  { not supported; ignored }
end;

function TProgressForm.DoesUserConfirmAction(Message : WideString; Default : Boolean) : Boolean;
begin
  Result := MessageDlg(Message, mtConfirmation, [mbYes, mbNo], 0)  = mrYes;
end;

procedure TProgressForm.ProcessMessages;
begin
  if HasUserInterface then begin
    Application.ProcessMessages;
  end;
end;

{ TOYProgressForm }
procedure TOYProgressForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style + WS_POPUP - WS_CHILD;
end;

procedure TOYProgressForm.CancelButtonClick(Sender: TObject);
begin
  if Assigned(fOnCancel) then begin
    fOnCancel(Self);
  end;
end;

procedure TOYProgressForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caNone;
end;

procedure TOYProgressForm.Button1Click(Sender: TObject);
begin
  if Assigned(fOnCancel) then begin
    fOnCancel(Self);
  end;
end;

initialization
  gDoHook := TList.Create;

finalization
  gDoHook.Free;

end.
