{
	Code Linguine Source Code Parsing/Query Library
	Copyright (C) 2000-2005 Pavel Simakov
	http://www.softwaresecretweapons.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
}

{$I oycl.inc}  


unit OYSyntaxView;

interface

uses { vcl } Windows, Controls, Messages, Graphics, Classes,
             StdCtrls, Menus, Grids,
     { oy } OYPackage;

type
  TSyntaxListBox = class(TDrawGrid)
  private
    fCharWidth : Integer;
    fGutWidth : Integer;
    fIHighlighter : IOYSyntaxHighlight;
    fFonts : TList;
    fPopup : TPopupMenu;
    fItems : TStrings;
    fStyles : TList;

  protected
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
    procedure NewDrawItem(Sender: TObject; ACol, ARow: Longint; Rect: TRect; State: TGridDrawState);
    function SelectCustomStyle(Items : TStrings; ItemIndex, CharIndex : Integer) : Integer;
    procedure FlipMenuItem(Sender : TObject);
    procedure FlipOnAllMenuItems(Sender : TObject);
    procedure FlipOffAllMenuItems(Sender : TObject);
    procedure IncreaseFont(Sender : TObject);
    procedure DecreaseFont(Sender : TObject);
    procedure Copy(Sender : TObject);
    procedure RecalculateView;
  public
    constructor Create(AOwner : TComponent; const IHighlighter : IOYSyntaxHighlight); reintroduce;
    destructor Destroy; override;
  public
    procedure LoadFromFile(const FileName : String);
    procedure SetTopLine(const LineNo : Integer);
  end;


implementation

uses { vcl } SysUtils, Math, Clipbrd;

{ TSyntaxListBox }
constructor TSyntaxListBox.Create(AOwner : TComponent; const IHighlighter : IOYSyntaxHighlight);
var
  i : Integer;
  aItem : TMenuItem;
begin
  inherited Create(AOwner);

  fIHighlighter := IHighlighter;

  fPopup := TPopupMenu.Create(Self);
  fFonts := TList.Create;
  fItems := TStringList.Create;
  fStyles := TList.Create;

  aItem := TMenuItem.Create(fPopup);
  aItem.Caption := 'Copy';
  aItem.Shortcut := ShortCut(Word('C'), [ssCtrl]);
  aItem.OnClick := Copy;
  fPopup.Items.Add(aItem);
  aItem := TMenuItem.Create(fPopup);
  aItem.Caption := '-';
  fPopup.Items.Add(aItem);

  { font style and item visibility items }
  if Assigned(fIHighlighter) then begin
    for i := 0 to fIHighlighter.IHighlightSchema.GetCount - 1 do begin
      aItem := TMenuItem.Create(fPopup);
      aItem.Checked := True;
      aItem.Caption := 'Show ' + fIHighlighter.IHighlightSchema.GetName(i);
      aItem.Shortcut := ShortCut(Word(49 + i), [ssCtrl]);
      aItem.OnClick := FlipMenuItem;
      fPopup.Items.Add(aItem);

      fStyles.Add(aItem);
      fFonts.Add(TFont.Create);
    end;

    if fIHighlighter.IHighlightSchema.GetCount <> 0 then begin
      aItem := TMenuItem.Create(fPopup);
      aItem.Caption := '-';
      fPopup.Items.Add(aItem);

      aItem := TMenuItem.Create(fPopup);
      aItem.Caption := 'Show All';
      aItem.OnClick := FlipOnAllMenuItems;
      fPopup.Items.Add(aItem);
      aItem := TMenuItem.Create(fPopup);
      aItem.Caption := 'Hide All';
      aItem.OnClick := FlipOffAllMenuItems;
      fPopup.Items.Add(aItem);

      aItem := TMenuItem.Create(fPopup);
      aItem.Caption := '-';
      fPopup.Items.Add(aItem);
    end;

  end;

  { other menues }
  aItem := TMenuItem.Create(fPopup);
  aItem.Caption := 'Increase Font Size';
  aItem.OnClick := IncreaseFont;
  aItem.Shortcut := ShortCut(Word('I'), [ssCtrl]);
  fPopup.Items.Add(aItem);
  aItem := TMenuItem.Create(fPopup);
  aItem.Caption := 'Decrease Font Size';
  aItem.OnClick := DecreaseFont;
  aItem.Shortcut := ShortCut(Word('D'), [ssCtrl]);
  fPopup.Items.Add(aItem);            
  PopupMenu := fPopup;
  
  { properties }
  OnDrawCell := NewDrawItem;
  GridLineWidth := 0;
  Options := Options + [goRangeSelect];
end;

destructor TSyntaxListBox.Destroy;
var
  i : Integer;
begin
  for i := 0 to fFonts.Count - 1 do begin
    TObject(fFonts[i]).Free;
  end;
  fFonts.Free;
  fStyles.Free;
  fItems.Free;

  inherited Destroy;
end;

procedure TSyntaxListBox.FlipMenuItem(Sender : TObject);
begin
  if Sender is TMenuItem then begin
    TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
    Invalidate;
  end;
end;

procedure TSyntaxListBox.FlipOnAllMenuItems(Sender : TObject);
var
  i : Integer;
begin
  for i := 0 to fStyles.Count - 1 do begin
    TMenuItem(fStyles[i]).Checked := True;
  end;
  Invalidate;
end;

procedure TSyntaxListBox.FlipOffAllMenuItems(Sender : TObject);
var
  i : Integer;
begin
  for i := 0 to fStyles.Count - 1 do begin
    TMenuItem(fStyles[i]).Checked := False;
  end;
  Invalidate;
end;

procedure TSyntaxListBox.IncreaseFont(Sender : TObject);
begin
  if Font.Size < 72 then begin
    Font.Size := Font.Size + 1;
  end;
  RecalculateView;
  Repaint;
end;

procedure TSyntaxListBox.DecreaseFont(Sender : TObject);
begin
  if Font.Size > 1 then begin
    Font.Size := Font.Size - 1;
  end;
  RecalculateView;
  Repaint;
end;

procedure TSyntaxListBox.Copy(Sender : TObject);
var
  i : Integer;
  aC : String;
begin
  aC := '';
  for i := Selection.Top to Selection.Bottom do begin
    aC := aC + fItems[i] + #$D#$A;
  end;
  Clipboard.AsText := aC;
end;

procedure TSyntaxListBox.WMEraseBkgnd(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TSyntaxListBox.LoadFromFile(const FileName : String);
begin
  fItems.LoadFromFile(FileName);
  RecalculateView;
  SetTopLine(1);
end;

procedure TSyntaxListBox.SetTopLine(const LineNo : Integer);
var
  aRect : TGridRect;
begin
  if LineNo > 1 then begin
    TopRow := LineNo - 1;
  end else begin
    TopRow := 0;
  end;
  aRect.Left := 0;
  aRect.Right := 0;
  aRect.Top := TopRow;
  aRect.Bottom := TopRow;
  Selection := aRect;
end;

procedure TSyntaxListBox.RecalculateView;
var
  i, aSz : Integer;
  tm : TTextMetric;
  aFont : TFont;
  aRect : TGridRect;
  aTop : Integer;
begin
  aTop := TopRow;
  aRect := Selection;
  try

    { cells }
    ColCount := 0;
    RowCount := 0;
    FixedCols := 0;
    FixedRows := 0;

    { fonts }
    Font.Color := clWindowText;
    Font.Style := [];
    Canvas.Font.Assign(Font);
    if GetTextMetrics(Canvas.Handle, tm) then begin
      DefaultRowHeight := tm.tmHeight;
    end;

    { scroler }
    RowCount := fItems.Count;
    aSz := 0;
    for i := 0 to fItems.Count - 1 do begin
      aSz := MaxIntValue( [ aSz, Canvas.TextWidth(IntToStr(fItems.Count + 1) + ' ' + fItems[i])] );
    end;
    DefaultColWidth := aSz;

    { customize fonts }
    for i := 0 to fFonts.Count - 1 do begin
      aFont := TFont(fFonts[i]);
      aFont.Assign(Canvas.Font);                        
      case i of
        1 : aFont.Style := aFont.Style + [fsBold];  // reserved
        2 : begin
          aFont.Style := aFont.Style + [fsBold];
          aFont.Color := clGreen;                   // literals
        end;
        3 : begin                                   // numbers
          aFont.Style := aFont.Style + [fsBold];
          aFont.Color := clRed;
        end;
        4 : aFont.Color := clBlue;                  // comments
        5 : aFont.Style := aFont.Style + [fsBold];  // punctuation
        6 : begin                                   // asm
          aFont.Style := aFont.Style + [fsBold];
          aFont.Color := clFuchsia;
        end;
        7 : begin                                   // directives
          aFont.Style := aFont.Style + [fsBold];
          aFont.Color := clLime;                                  
        end;
      end;
    end;

    { gutter }
    fGutWidth := Canvas.TextWidth(IntToStr(fItems.Count + 1) + ' ');

    { chars }
    fCharWidth := Canvas.TextWidth('W');

  finally
    Selection := aRect;
    TopRow := aTop;
  end;
end;                                                   

function TSyntaxListBox.SelectCustomStyle(Items : TStrings; ItemIndex, CharIndex : Integer) : Integer;
begin
  if Assigned(fIHighlighter) then begin
    Result := fIHighlighter.GetTokenIndexInSchema(ItemIndex + 1, CharIndex + 1);
  end else begin
    Result := -1;
  end;
end;

procedure TSyntaxListBox.NewDrawItem(Sender: TObject; ACol, ARow: Longint; Rect: TRect; State: TGridDrawState);
var
  i : Integer;
  aChar : String;
  aRect : TRect;
  aAlign : Integer;
  aCache : String;
  aPen : HGDIOBJ;
  aStyle, aDefault, aCacheStyle : Integer;

  procedure _PaintFragment;
  var
    aFont : HGDIOBJ;
    aColor : HGDIOBJ;
    aP : Pointer;
    aL : LONGWORD;
  begin
    { paint current line fragment and update rect }
    if Length(aCache) > 0 then begin
      if (aCacheStyle >= 0) then begin
        if (TMenuItem(fStyles[aCacheStyle]).Checked) then begin
          aColor := SetTextColor(Canvas.Handle, ColorToRgb(TFont(fFonts[aCacheStyle]).Color));
          aFont := SelectObject(Canvas.Handle, TFont(fFonts[aCacheStyle]).Handle);
        end else begin
          aColor := SetTextColor(Canvas.Handle, ColorToRgb(clWindow));
          aFont := SelectObject(Canvas.Handle, Canvas.Font.Handle);
        end;
      end else begin
        aColor := SetTextColor(Canvas.Handle, ColorToRgb(Canvas.Font.Color));
        aFont := SelectObject(Canvas.Handle, Canvas.Font.Handle);
      end;
      aP := nil;
      aL := TabbedTextOut(Canvas.Handle, aRect.Left, aRect.Top, PChar(aCache), Length(aCache), 0, aP, 0);
      aRect.Left := aRect.Left + LoWord(aL);
      SelectObject(Canvas.Handle, aFont);
      SetTextColor(Canvas.Handle, aColor);
      aCache := '';
    end;
  end;

var
  tBrush : TColor;
  tFont : TColor;
  cFont : TColor;
  cBrush : TColor;

  procedure _InitColors;
  begin
    { gutter }
    if gdSelected in State  then begin
      tBrush := clHighlight;
    end else begin            
      tBrush := clBtnFace;
    end;
    if gdSelected in State  then begin
      tFont := clHighlightText;
    end else begin
      tFont := clBtnText;
    end;

    { client area }
    if gdSelected in State  then begin
      cBrush := cl3DLight;                
    end else begin         
      cBrush := clWindow;
    end;
    if gdSelected in State  then begin
      cFont := clBtnText;
    end else begin
      cFont := clBtnText;
    end;
  end;

begin
  _InitColors;

  { gutter }
  aRect := Rect;
  aRect.Right := aRect.Left + fGutWidth;
  Canvas.Brush.Color := tBrush;;
  FillRect(Canvas.Handle, aRect, Canvas.Brush.Handle);
  Canvas.Font.Color := tFont;
  aChar := IntToStr(aRow + 1);
  aAlign := SetTextAlign(Canvas.Handle, TA_RIGHT);
  TextOut(Canvas.Handle, aRect.Right - fCharWidth div 2, aRect.Top, PChar(aChar), Length(aChar));
  SetTextAlign(Canvas.Handle, aAlign);

  { paint non-client area at the start of the line }
  aRect := Rect;
  aRect.Left := aRect.Left + fGutWidth - fCharWidth div 2;
  aRect.Right := aRect.Left + fGutWidth;
  Canvas.Brush.Color := cBrush;
  FillRect(Canvas.Handle, aRect, Canvas.Brush.Handle);

  { draw line separator }
  aRect := Rect;
  Canvas.Pen.Color := cl3DLight;
  aRect.Left := 1 + aRect.Left + fGutWidth - fCharWidth div 2;
  aPen := SelectObject(Canvas.Handle, Canvas.Pen.Handle);
  MoveToEx(Canvas.Handle, aRect.Left,  aRect.Top, nil);
  LineTo(Canvas.Handle, aRect.Left,  aRect.Bottom);
  SelectObject(Canvas.Handle, aPen);

  { paint text char by char }
  Canvas.Font.Color := cFont;
  aRect := Rect;
  aRect.Left := aRect.Left + fGutWidth;
  aDefault := fFonts.Count - 1;
  aCacheStyle := aDefault;
  for i := 1 to Length(fItems[aRow]) do begin
    aStyle := SelectCustomStyle(fItems, aRow, i);
    if (aStyle <> aCacheStyle) then begin
      _PaintFragment;
      aCacheStyle := aStyle;
    end;
    aCache := aCache + fItems[aRow][i];
  end;
  _PaintFragment;

  { paint non-client area at the end of the line }
  aRect.Right := aRect.Left + ClientWidth; 
  Canvas.Brush.Color := cBrush;
  FillRect(Canvas.Handle, aRect, Canvas.Brush.Handle);

  { paint non-client area at the bottom }
  if aRow = fItems.Count - 1 then begin
    aRect := Rect;
    aRect.Top := aRect.Bottom;
    aRect.Bottom := ClientRect.Bottom;
    Canvas.Brush.Color := cBrush;
    FillRect(Canvas.Handle, aRect, Canvas.Brush.Handle);
  end;
end;

end.
